/* File: tables.c */

/*
 * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
 *
 * This software may be copied and distributed for educational, research,
 * and not for profit purposes provided that this copyright and statement
 * are included in all such copies.  Other copyrights may also apply.
 *
 * UnAngband (c) 2001-2009 Andrew Doull. Modifications to the Angband 2.9.1
 * source code are released under the Gnu Public License. See www.fsf.org
 * for current GPL license details. Addition permission granted to
 * incorporate modifications in all Angband variants as defined in the
 * Angband variants FAQ. See rec.games.roguelike.angband for FAQ.
 */

#include "angband.h"




/*
 * Global array for looping through the "keypad directions".
 */
const s16b ddd[9] =
{ 2, 8, 6, 4, 3, 1, 9, 7, 5 };

/*
 * Global arrays for converting "keypad direction" into "offsets".
 */
const s16b ddx[10] =
{ 0, -1, 0, 1, -1, 0, 1, -1, 0, 1 };

const s16b ddy[10] =
{ 0, 1, 1, 1, 0, 0, 0, -1, -1, -1 };

/*
 * Global arrays for optimizing "ddx[ddd[i]]" and "ddy[ddd[i]]".
 */
const s16b ddx_ddd[9] =
{ 0, 0, 1, -1, 1, -1, 1, -1, 0 };

const s16b ddy_ddd[9] =
{ 1, -1, 0, 0, 1, 1, -1, -1, 0 };


/*
 * Given a central direction at position [dir #][0], return a series
 * of directions radiating out on both sides from the central direction
 * all the way back to its rear.
 *
 * Side directions come in pairs; for example, directions '1' and '3'
 * flank direction '2'.  The code should know which side to consider
 * first.  If the left, it must add 10 to the central direction to
 * access the second part of the table.
 */
const byte side_dirs[20][8] =
{
	{ 0, 0, 0, 0, 0, 0, 0, 0 },	/* bias right */
	{ 1, 4, 2, 7, 3, 8, 6, 9 },
	{ 2, 1, 3, 4, 6, 7, 9, 8 },
	{ 3, 2, 6, 1, 9, 4, 8, 7 },
	{ 4, 7, 1, 8, 2, 9, 3, 6 },
	{ 5, 5, 5, 5, 5, 5, 5, 5 },
	{ 6, 3, 9, 2, 8, 1, 7, 4 },
	{ 7, 8, 4, 9, 1, 6, 2, 3 },
	{ 8, 9, 7, 6, 4, 3, 1, 2 },
	{ 9, 6, 8, 3, 7, 2, 4, 1 },

	{ 0, 0, 0, 0, 0, 0, 0, 0 },	/* bias left */
	{ 1, 2, 4, 3, 7, 6, 8, 9 },
	{ 2, 3, 1, 6, 4, 9, 7, 8 },
	{ 3, 6, 2, 9, 1, 8, 4, 7 },
	{ 4, 1, 7, 2, 8, 3, 9, 6 },
	{ 5, 5, 5, 5, 5, 5, 5, 5 },
	{ 6, 9, 3, 8, 2, 7, 1, 4 },
	{ 7, 4, 8, 1, 9, 2, 6, 3 },
	{ 8, 7, 9, 4, 6, 1, 3, 2 },
	{ 9, 8, 6, 7, 3, 4, 2, 1 }
};


/*
 * Global array for converting numbers to uppercase hecidecimal digit
 * This array can also be used to convert a number to an octal digit
 */
const char hexsym[16] =
{
	'0', '1', '2', '3', '4', '5', '6', '7',
	'8', '9', 'A', 'B', 'C', 'D', 'E', 'F'
};



/*
 * Global array for number of stats to increase at each level.
 * Ensure that no number here is greater than PY_MAX_STAT_GAIN.
 */
const byte stat_gains[PY_MAX_LEVEL] =
{
/* This spreads the stat gains better at lower levels and
	gives bigger increases at higher levels. */
	0, 2, 0, 0, 2, 0, 0, 0, 0, 5,
	0, 0, 2, 0, 0, 2, 0, 2, 0, 5,
	0, 2, 0, 2, 0, 2, 0, 2, 0, 5,
	0, 2, 2, 2, 2, 2, 2, 2, 2, 5,
	2, 2, 2, 2, 2, 2, 3, 3, 3, 5
};


/*
 * Stat Table -- One less than number of spells at adj_mag_study_max (see below).
 *
 * This attempts to balance out the maximum number of spells available, which we
 * do not want to be too high at high levels, as this is unbalancing, against
 * the rate of spells learnt, which is important at low levels.
 */
const byte adj_mag_study[] =
{
	6	/* 3 */,
	8	/* 4 */,
	10	/* 5 */,
	12	/* 6 */,
	15	/* 7 */,
	20	/* 8 */,
	25	/* 9 */,
	30	/* 10 */,
	35	/* 11 */,
	40	/* 12 */,
	42	/* 13 */,
	45	/* 14 */,
	47	/* 15 */,
	50	/* 16 */,
	51	/* 17 */,
	52	/* 18/00-18/09 */,
	53	/* 18/10-18/19 */,
	54	/* 18/20-18/29 */,
	55	/* 18/30-18/39 */,
	56	/* 18/40-18/49 */,
	57	/* 18/50-18/59 */,
	58	/* 18/60-18/69 */,
	59	/* 18/70-18/79 */,
	60	/* 18/80-18/89 */,
	62	/* 18/90-18/99 */,
	64	/* 18/100-18/109 */,
	66	/* 18/110-18/119 */,
	68	/* 18/120-18/129 */,
	70	/* 18/130-18/139 */,
	73	/* 18/140-18/149 */,
	76	/* 18/150-18/159 */,
	80	/* 18/160-18/169 */,
	84	/* 18/170-18/179 */,
	88	/* 18/180-18/189 */,
	92	/* 18/190-18/199 */,
	96	/* 18/200-18/209 */,
	100	/* 18/210-18/219 */,
	105	/* 18/220-18/229 */,
	110	/* 18/230-18/239 */,
	115	/* 18/240-18/249 */,
	120	/* 18/250+ */
};


/*
 * Stat Table -- Maximum level which caster stops learning additional spells.
 */
const byte adj_mag_study_max[] =
{
	18	/* 3 */,
	24	/* 4 */,
	28	/* 5 */,
	32	/* 6 */,
	36	/* 7 */,
	40	/* 8 */,
	42	/* 9 */,
	44	/* 10 */,
	45	/* 11 */,
	46	/* 12 */,
	47	/* 13 */,
	48	/* 14 */,
	49	/* 15 */,
	50	/* 16 */,
	48	/* 17 */,
	46	/* 18/00-18/09 */,
	44	/* 18/10-18/19 */,
	42	/* 18/20-18/29 */,
	40	/* 18/30-18/39 */,
	38	/* 18/40-18/49 */,
	36	/* 18/50-18/59 */,
	34	/* 18/60-18/69 */,
	32	/* 18/60-18/69 */,
	30	/* 18/70-18/79 */,
	30	/* 18/80-18/89 */,
	30	/* 18/90-18/99 */,
	30	/* 18/100-18/109 */,
	30	/* 18/110-18/119 */,
	30	/* 18/120-18/129 */,
	30	/* 18/130-18/139 */,
	30	/* 18/140-18/149 */,
	30	/* 18/150-18/159 */,
	31	/* 18/160-18/169 */,
	32	/* 18/170-18/179 */,
	33	/* 18/180-18/189 */,
	34	/* 18/190-18/199 */,
	35	/* 18/200-18/209 */,
	36	/* 18/210-18/219 */,
	37	/* 18/220-18/229 */,
	38	/* 18/230-18/239 */,
	39	/* 18/240-18/249 */,
	40	/* 18/250+ */
};


/*
 * Stat Table -- one (or 4 for mages) less than the amount of mana at level 50
 *
 * Now also used for 'reach' with size.
 */
const s16b adj_mag_mana[] =
{
	0	/* 3 */,
	5	/* 4 */,
	10	/* 5 */,
	15	/* 6 */,
	20	/* 7 */,
	25	/* 8 */,
	30	/* 9 */,
	35	/* 10 */,
	40	/* 11 */,
	45	/* 12 */,
	50	/* 13 */,
	55	/* 14 */,
	60	/* 15 */,
	65	/* 16 */,
	70	/* 17 */,
	75	/* 18/00-18/09 */,
	80	/* 18/10-18/19 */,
	85	/* 18/20-18/29 */,
	90	/* 18/30-18/39 */,
	95	/* 18/40-18/49 */,
	100	/* 18/50-18/59 */,
	110	/* 18/60-18/69 */,
	120	/* 18/70-18/79 */,
	130	/* 18/80-18/89 */,
	140	/* 18/90-18/99 */,
	160	/* 18/100-18/109 */,
	180	/* 18/110-18/119 */,
	200	/* 18/120-18/129 */,
	220	/* 18/130-18/139 */,
	240	/* 18/140-18/149 */,
	260	/* 18/150-18/159 */,
	280	/* 18/160-18/169 */,
	300	/* 18/170-18/179 */,
	320	/* 18/180-18/189 */,
	340	/* 18/190-18/199 */,
	360	/* 18/200-18/209 */,
	380	/* 18/210-18/219 */,
	400	/* 18/220-18/229 */,
	410	/* 18/230-18/239 */,
	415	/* 18/240-18/249 */,
	420	/* 18/250+ */
};


/*
 * Stat Table (spell stat) -- Minimum failure rate (percentage)
 */
const byte adj_mag_fail_min[] =
{
	99	/* 3 */,
	99	/* 4 */,
	99	/* 5 */,
	99	/* 6 */,
	99	/* 7 */,
	50	/* 8 */,
	30	/* 9 */,
	20	/* 10 */,
	15	/* 11 */,
	12	/* 12 */,
	11	/* 13 */,
	10	/* 14 */,
	9	/* 15 */,
	8	/* 16 */,
	7	/* 17 */,
	6	/* 18/00-18/09 */,
	6	/* 18/10-18/19 */,
	5	/* 18/20-18/29 */,
	5	/* 18/30-18/39 */,
	5	/* 18/40-18/49 */,
	4	/* 18/50-18/59 */,
	4	/* 18/60-18/69 */,
	4	/* 18/70-18/79 */,
	3	/* 18/80-18/89 */,
	3	/* 18/90-18/99 */,
	3	/* 18/100-18/109 */,
	2	/* 18/110-18/119 */,
	2	/* 18/120-18/129 */,
	2	/* 18/130-18/139 */,
	2	/* 18/140-18/149 */,
	1	/* 18/150-18/159 */,
	1	/* 18/160-18/169 */,
	1	/* 18/170-18/179 */,
	1	/* 18/180-18/189 */,
	1	/* 18/190-18/199 */,
	0	/* 18/200-18/209 */,
	0	/* 18/210-18/219 */,
	0	/* 18/220-18/229 */,
	0	/* 18/230-18/239 */,
	0	/* 18/240-18/249 */,
	0	/* 18/250+ */
};


/*
 * Stat Table (spell stat) -- Rate fail
 */
const byte adj_mag_fail_rate[] =
{
	0	/* 3 */,
	0	/* 4 */,
	0	/* 5 */,
	0	/* 6 */,
	0	/* 7 */,
	1	/* 8 */,
	1	/* 9 */,
	1	/* 10 */,
	1	/* 11 */,
	1	/* 12 */,
	2	/* 13 */,
	2	/* 14 */,
	2	/* 15 */,
	2	/* 16 */,
	2	/* 17 */,
	3	/* 18/00-18/09 */,
	3	/* 18/10-18/19 */,
	3	/* 18/20-18/29 */,
	3	/* 18/30-18/39 */,
	3	/* 18/40-18/49 */,
	4	/* 18/50-18/59 */,
	4	/* 18/60-18/69 */,
	5	/* 18/70-18/79 */,
	6	/* 18/80-18/89 */,
	7	/* 18/90-18/99 */,
	8	/* 18/100-18/109 */,
	9	/* 18/110-18/119 */,
	10	/* 18/120-18/129 */,
	11	/* 18/130-18/139 */,
	12	/* 18/140-18/149 */,
	13	/* 18/150-18/159 */,
	14	/* 18/160-18/169 */,
	15	/* 18/170-18/179 */,
	16	/* 18/180-18/189 */,
	17	/* 18/190-18/199 */,
	18	/* 18/200-18/209 */,
	19	/* 18/210-18/219 */,
	20	/* 18/220-18/229 */,
	21	/* 18/230-18/239 */,
	21	/* 18/240-18/249 */,
	22	/* 18/250+ */
};


/*
 * Stat Table (CHR) -- payment percentages
 */
const byte adj_chr_gold[] =
{
	130	/* 3 */,
	125	/* 4 */,
	122	/* 5 */,
	120	/* 6 */,
	118	/* 7 */,
	116	/* 8 */,
	114	/* 9 */,
	112	/* 10 */,
	110	/* 11 */,
	108	/* 12 */,
	106	/* 13 */,
	104	/* 14 */,
	103	/* 15 */,
	102	/* 16 */,
	101	/* 17 */,
	100	/* 18/00-18/09 */,
	99	/* 18/10-18/19 */,
	98	/* 18/20-18/29 */,
	97	/* 18/30-18/39 */,
	96	/* 18/40-18/49 */,
	95	/* 18/50-18/59 */,
	94	/* 18/60-18/69 */,
	93	/* 18/70-18/79 */,
	92	/* 18/80-18/89 */,
	91	/* 18/90-18/99 */,
	90	/* 18/100-18/109 */,
	89	/* 18/110-18/119 */,
	88	/* 18/120-18/129 */,
	87	/* 18/130-18/139 */,
	86	/* 18/140-18/149 */,
	85	/* 18/150-18/159 */,
	84	/* 18/160-18/169 */,
	83	/* 18/170-18/179 */,
	82	/* 18/180-18/189 */,
	81	/* 18/190-18/199 */,
	80	/* 18/200-18/209 */,
	79	/* 18/210-18/219 */,
	78	/* 18/220-18/229 */,
	77	/* 18/230-18/239 */,
	77	/* 18/240-18/249 */,
	76	/* 18/250+ */
};


/*
 * Stat Table (CHR) -- chance of restocking an item out of stock
 */
const byte adj_chr_stock[] =
{
	230	/* 3 */,
	220	/* 4 */,
	215	/* 5 */,
	210	/* 6 */,
	205	/* 7 */,
	200	/* 8 */,
	195	/* 9 */,
	190	/* 10 */,
	185	/* 11 */,
	180	/* 12 */,
	175	/* 13 */,
	160	/* 14 */,
	155	/* 15 */,
	140	/* 16 */,
	135	/* 17 */,
	130	/* 18/00-18/09 */,
	120	/* 18/10-18/19 */,
	110	/* 18/20-18/29 */,
	100	/* 18/30-18/39 */,
	90	/* 18/40-18/49 */,
	80	/* 18/50-18/59 */,
	70	/* 18/60-18/69 */,
	60	/* 18/70-18/79 */,
	55	/* 18/80-18/89 */,
	51	/* 18/90-18/99 */,
	48	/* 18/100-18/109 */,
	46	/* 18/110-18/119 */,
	44	/* 18/120-18/129 */,
	42	/* 18/130-18/139 */,
	40	/* 18/140-18/149 */,
	38	/* 18/150-18/159 */,
	36	/* 18/160-18/169 */,
	34	/* 18/170-18/179 */,
	32	/* 18/180-18/189 */,
	30	/* 18/190-18/199 */,
	28	/* 18/200-18/209 */,
	26	/* 18/210-18/219 */,
	24	/* 18/220-18/229 */,
	23	/* 18/230-18/239 */,
	23	/* 18/240-18/249 */,
	22	/* 18/250+ */
};


/*
 * Stat Table (CHR) -- Taunt -- save vs. monster AI using ranged attack
 */
const byte adj_chr_taunt[] =
{
	90	/* 3 */,
	91	/* 4 */,
	92	/* 5 */,
	93	/* 6 */,
	94	/* 7 */,
	95	/* 8 */,
	96	/* 9 */,
	97	/* 10 */,
	98	/* 11 */,
	99	/* 12 */,
	100	/* 13 */,
	100	/* 14 */,
	101	/* 15 */,
	101	/* 16 */,
	101	/* 17 */,
	102	/* 18/00-18/09 */,
	102	/* 18/10-18/19 */,
	102	/* 18/20-18/29 */,
	103	/* 18/30-18/39 */,
	103	/* 18/40-18/49 */,
	104	/* 18/50-18/59 */,
	104	/* 18/60-18/69 */,
	105	/* 18/70-18/79 */,
	105	/* 18/80-18/89 */,
	106	/* 18/90-18/99 */,
	107	/* 18/100-18/109 */,
	108	/* 18/110-18/119 */,
	119	/* 18/120-18/129 */,
	110	/* 18/130-18/139 */,
	111	/* 18/140-18/149 */,
	112	/* 18/150-18/159 */,
	113	/* 18/160-18/169 */,
	114	/* 18/170-18/179 */,
	115	/* 18/180-18/189 */,
	116	/* 18/190-18/199 */,
	117	/* 18/200-18/209 */,
	118	/* 18/210-18/219 */,
	119	/* 18/220-18/229 */,
	120	/* 18/230-18/239 */,
	120	/* 18/240-18/249 */,
	121	/* 18/250+ */
};


/*
 * Stat Table from 1 to 31; this is a bad idea, better drop "const"
 */
#define StatTable_1_31 \
{\
	0	/* 3 */,\
	1	/* 4 */,\
	2	/* 5 */,\
	3	/* 6 */,\
	4	/* 7 */,\
	5	/* 8 */,\
	6	/* 9 */,\
	7	/* 10 */,\
	8	/* 11 */,\
	9	/* 12 */,\
	10	/* 13 */,\
	10	/* 14 */,\
	10	/* 15 */,\
	11	/* 16 */,\
	11	/* 17 */,\
	12	/* 18/00-18/09 */,\
	12	/* 18/10-18/19 */,\
	13	/* 18/20-18/29 */,\
	13	/* 18/30-18/39 */,\
	13	/* 18/40-18/49 */,\
	14	/* 18/50-18/59 */,\
	14	/* 18/60-18/69 */,\
	15	/* 18/70-18/79 */,\
	15	/* 18/80-18/89 */,\
	16	/* 18/90-18/99 */,\
	17	/* 18/100-18/109 */,\
	18	/* 18/110-18/119 */,\
	19	/* 18/120-18/129 */,\
	20	/* 18/130-18/139 */,\
	21	/* 18/140-18/149 */,\
	22	/* 18/150-18/159 */,\
	23	/* 18/160-18/169 */,\
	24	/* 18/170-18/179 */,\
	25	/* 18/180-18/189 */,\
	26	/* 18/190-18/199 */,\
	27	/* 18/200-18/209 */,\
	28	/* 18/210-18/219 */,\
	29	/* 18/220-18/229 */,\
	30	/* 18/230-18/239 */,\
	30	/* 18/240-18/249 */,\
	31	/* 18/250+ */\
};


/*
 * Stat Table (CHR) -- Fear
 */
const byte adj_chr_fear[] = StatTable_1_31


/*
 * Stat Table (INT) -- Breakage chance for ammo and weapons
 */
const byte adj_int_break[] = StatTable_1_31


/*
 * Stat Table (INT) -- Magic devices
 */
const byte adj_int_dev[] =
{
	0 	/* 3 */,
	0 	/* 4 */,
	0 	/* 5 */,
	0 	/* 6 */,
	0 	/* 7 */,
	1 	/* 8 */,
	2 	/* 9 */,
	3 	/* 10 */,
	4 	/* 11 */,
	5 	/* 12 */,
	6 	/* 13 */,
	7 	/* 14 */,
	8 	/* 15 */,
	9 	/* 16 */,
	10 	/* 17 */,
	11 	/* 18/00-18/09 */,
	12 	/* 18/10-18/19 */,
	13 	/* 18/20-18/29 */,
	14 	/* 18/30-18/39 */,
	15 	/* 18/40-18/49 */,
	16 	/* 18/50-18/59 */,
	17 	/* 18/60-18/69 */,
	18 	/* 18/70-18/79 */,
	19	/* 18/80-18/89 */,
	20	/* 18/90-18/99 */,
	21 	/* 18/100-18/109 */,
	22 	/* 18/110-18/119 */,
	23	/* 18/120-18/129 */,
	24	/* 18/130-18/139 */,
	25	/* 18/140-18/149 */,
	26	/* 18/150-18/159 */,
	27	/* 18/160-18/169 */,
	28	/* 18/170-18/179 */,
	29	/* 18/180-18/189 */,
	30	/* 18/190-18/199 */,
	31	/* 18/200-18/209 */,
	32	/* 18/210-18/219 */,
	33	/* 18/220-18/229 */,
	35	/* 18/230-18/239 */,
	37	/* 18/240-18/249 */,
	40	/* 18/250+ */
};


/*
 * Stat Table (WIS) -- Saving throw
 */
const byte adj_wis_sav[] =
{
	0 	/* 3 */,
	1 	/* 4 */,
	1 	/* 5 */,
	2 	/* 6 */,
	2 	/* 7 */,
	3 	/* 8 */,
	3 	/* 9 */,
	4 	/* 10 */,
	4 	/* 11 */,
	5 	/* 12 */,
	5 	/* 13 */,
	6 	/* 14 */,
	6 	/* 15 */,
	7 	/* 16 */,
	7 	/* 17 */,
	8 	/* 18/00-18/09 */,
	9	/* 18/10-18/19 */,
	10 	/* 18/20-18/29 */,
	11 	/* 18/30-18/39 */,
	12 	/* 18/40-18/49 */,
	14 	/* 18/50-18/59 */,
	15 	/* 18/60-18/69 */,
	17 	/* 18/70-18/79 */,
	18 	/* 18/80-18/89 */,
	19 	/* 18/90-18/99 */,
	21 	/* 18/100-18/109 */,
	22 	/* 18/110-18/119 */,
	23	/* 18/120-18/129 */,
	24	/* 18/130-18/139 */,
	25	/* 18/140-18/149 */,
	27	/* 18/150-18/159 */,
	28	/* 18/160-18/169 */,
	29	/* 18/170-18/179 */,
	30	/* 18/180-18/189 */,
	31	/* 18/190-18/199 */,
	33	/* 18/200-18/209 */,
	34	/* 18/210-18/219 */,
	35	/* 18/220-18/229 */,
	37	/* 18/230-18/239 */,
	38	/* 18/240-18/249 */,
	40	/* 18/250+ */
};


/*
 * Stat Table (DEX) -- disarming
 */
const byte adj_dex_dis[] =
{
	0	/* 3 */,
	0	/* 4 */,
	0	/* 5 */,
	0	/* 6 */,
	0	/* 7 */,
	1	/* 8 */,
	1	/* 9 */,
	1	/* 10 */,
	1	/* 11 */,
	1	/* 12 */,
	2	/* 13 */,
	2	/* 14 */,
	2	/* 15 */,
	2	/* 16 */,
	2	/* 17 */,
	3	/* 18/00-18/09 */,
	3	/* 18/10-18/19 */,
	3	/* 18/20-18/29 */,
	4	/* 18/30-18/39 */,
	4	/* 18/40-18/49 */,
	4	/* 18/50-18/59 */,
	4	/* 18/60-18/69 */,
	5	/* 18/70-18/79 */,
	5	/* 18/80-18/89 */,
	5	/* 18/90-18/99 */,
	6	/* 18/100-18/109 */,
	6	/* 18/110-18/119 */,
	6	/* 18/120-18/129 */,
	7	/* 18/130-18/139 */,
	7	/* 18/140-18/149 */,
	7	/* 18/150-18/159 */,
	8	/* 18/160-18/169 */,
	8	/* 18/170-18/179 */,
	8	/* 18/180-18/189 */,
	9	/* 18/190-18/199 */,
	9	/* 18/200-18/209 */,
	9	/* 18/210-18/219 */,
	10	/* 18/220-18/229 */,
	10	/* 18/230-18/239 */,
	10	/* 18/240-18/249 */,
	11	/* 18/250+ */
};


/*
 * Stat Table (INT) -- disarming
 */
const byte adj_int_dis[] =
{
	0 	/* 3 */,
	0 	/* 4 */,
	0 	/* 5 */,
	0 	/* 6 */,
	0 	/* 7 */,
	1 	/* 8 */,
	1 	/* 9 */,
	1 	/* 10 */,
	1 	/* 11 */,
	1 	/* 12 */,
	2 	/* 13 */,
	2 	/* 14 */,
	2 	/* 15 */,
	2 	/* 16 */,
	2 	/* 17 */,
	3 	/* 18/00-18/09 */,
	3 	/* 18/10-18/19 */,
	3 	/* 18/20-18/29 */,
	3 	/* 18/30-18/39 */,
	3 	/* 18/40-18/49 */,
	4 	/* 18/50-18/59 */,
	4 	/* 18/60-18/69 */,
	5 	/* 18/70-18/79 */,
	6 	/* 18/80-18/89 */,
	7 	/* 18/90-18/99 */,
	8 	/* 18/100-18/109 */,
	9 	/* 18/110-18/119 */,
	10	/* 18/120-18/129 */,
	11	/* 18/130-18/139 */,
	12	/* 18/140-18/149 */,
	13	/* 18/150-18/159 */,
	14	/* 18/160-18/169 */,
	15	/* 18/170-18/179 */,
	16	/* 18/180-18/189 */,
	17	/* 18/190-18/199 */,
	18	/* 18/200-18/209 */,
	19	/* 18/210-18/219 */,
	20	/* 18/220-18/229 */,
	21	/* 18/230-18/239 */,
	21	/* 18/240-18/249 */,
	22	/* 18/250+ */
};


/*
 * Stat Table (AGI) -- bonus to ac (plus 128)
 */
const byte adj_agi_ta[] =
{
    128 + -4 /* 3 */,
    128 + -3 /* 4 */,
    128 + -2 /* 5 */,
    128 + -1 /* 6 */,
    128 + 0  /* 7 */,
    128 + 1  /* 8 */,
    128 + 1  /* 9 */,
    128 + 1  /* 10 */,
    128 + 1  /* 11 */,
    128 + 1  /* 12 */,
    128 + 2  /* 13 */,
    128 + 2  /* 14 */,
    128 + 2  /* 15 */,
    128 + 2  /* 16 */,
    128 + 2  /* 17 */,
    128 + 3  /* 18/00-18/09 */,
    128 + 3  /* 18/10-18/19 */,
    128 + 3  /* 18/20-18/29 */,
    128 + 3  /* 18/30-18/39 */,
    128 + 3  /* 18/40-18/49 */,
    128 + 4  /* 18/50-18/59 */,
    128 + 4  /* 18/60-18/69 */,
    128 + 5  /* 18/70-18/79 */,
    128 + 6  /* 18/80-18/89 */,
    128 + 7  /* 18/90-18/99 */,
    128 + 8  /* 18/100-18/109 */,
    128 + 9  /* 18/110-18/119 */,
    128 + 10 /* 18/120-18/129 */,
    128 + 11 /* 18/130-18/139 */,
    128 + 12 /* 18/140-18/149 */,
    128 + 13 /* 18/150-18/159 */,
    128 + 14 /* 18/160-18/169 */,
    128 + 15 /* 18/170-18/179 */,
    128 + 16 /* 18/180-18/189 */,
    128 + 17 /* 18/190-18/199 */,
    128 + 18 /* 18/200-18/209 */,
    128 + 19 /* 18/210-18/219 */,
    128 + 20 /* 18/220-18/229 */,
    128 + 21 /* 18/230-18/239 */,
    128 + 21 /* 18/240-18/249 */,
    128 + 22 /* 18/250+ */
};


/*
 * Stat Table (WIS) -- bonus to ac (plus 128)
 */
const byte adj_wis_ta[] =
{
    128 + -8 /* 3 */,
    128 + -7 /* 4 */,
    128 + -6 /* 5 */,
    128 + -5 /* 6 */,
    128 + -4 /* 7 */,
    128 + -3 /* 8 */,
    128 + -2 /* 9 */,
    128 + -1 /* 10 */,
    128 + 0  /* 11 */,
    128 + 0  /* 12 */,
    128 + 0  /* 13 */,
    128 + 0  /* 14 */,
    128 + 0  /* 15 */,
    128 + 1  /* 16 */,
    128 + 2  /* 17 */,
    128 + 3  /* 18/00-18/09 */,
    128 + 4  /* 18/10-18/19 */,
    128 + 5  /* 18/20-18/29 */,
    128 + 6  /* 18/30-18/39 */,
    128 + 7  /* 18/40-18/49 */,
    128 + 8  /* 18/50-18/59 */,
    128 + 9  /* 18/60-18/69 */,
    128 + 10 /* 18/70-18/79 */,
    128 + 11 /* 18/80-18/89 */,
    128 + 12 /* 18/90-18/99 */,
    128 + 13 /* 18/100-18/109 */,
    128 + 14 /* 18/110-18/119 */,
    128 + 15 /* 18/120-18/129 */,
    128 + 16 /* 18/130-18/139 */,
    128 + 17 /* 18/140-18/149 */,
    128 + 18 /* 18/150-18/159 */,
    128 + 19 /* 18/160-18/169 */,
    128 + 20 /* 18/170-18/179 */,
    128 + 21 /* 18/180-18/189 */,
    128 + 22 /* 18/190-18/199 */,
    128 + 23 /* 18/200-18/209 */,
    128 + 24 /* 18/210-18/219 */,
    128 + 25 /* 18/220-18/239 */,
    128 + 26 /* 18/230-18/249 */,
    128 + 26 /* 18/240-18/249 */,
    128 + 27 /* 18/250+ */
};


/*
 * Stat Table (STR) -- bonus to dam (plus 128)
 */
const byte adj_str_td[] =
{
	128 + -2	/* 3 */,
	128 + -2	/* 4 */,
	128 + -1	/* 5 */,
	128 + -1	/* 6 */,
	128 + 0	/* 7 */,
	128 + 0	/* 8 */,
	128 + 0	/* 9 */,
	128 + 0	/* 10 */,
	128 + 0	/* 11 */,
	128 + 0	/* 12 */,
	128 + 0	/* 13 */,
	128 + 0	/* 14 */,
	128 + 0	/* 15 */,
	128 + 1	/* 16 */,
	128 + 2	/* 17 */,
	128 + 2	/* 18/00-18/09 */,
	128 + 2	/* 18/10-18/19 */,
	128 + 3	/* 18/20-18/29 */,
	128 + 3	/* 18/30-18/39 */,
	128 + 3	/* 18/40-18/49 */,
	128 + 4	/* 18/50-18/59 */,
	128 + 4	/* 18/60-18/69 */,
	128 + 4	/* 18/70-18/79 */,
	128 + 5	/* 18/80-18/89 */,
	128 + 5	/* 18/90-18/99 */,
	128 + 6	/* 18/100-18/109 */,
	128 + 7	/* 18/110-18/119 */,
	128 + 8	/* 18/120-18/129 */,
	128 + 9	/* 18/130-18/139 */,
	128 + 10	/* 18/140-18/149 */,
	128 + 11	/* 18/150-18/159 */,
	128 + 12	/* 18/160-18/169 */,
	128 + 13	/* 18/170-18/179 */,
	128 + 14	/* 18/180-18/189 */,
	128 + 15	/* 18/190-18/199 */,
	128 + 16	/* 18/200-18/209 */,
	128 + 18	/* 18/210-18/219 */,
	128 + 20	/* 18/220-18/229 */,
	128 + 21	/* 18/230-18/239 */,
	128 + 22	/* 18/240-18/249 */,
	128 + 23	/* 18/250+ */
};


/*
 * Stat Table (DEX) -- bonus to hit (plus 128)
 */
const byte adj_dex_th[] =
{
    128 + -8 /* 3 */,
    128 + -7 /* 4 */,
    128 + -6 /* 5 */,
    128 + -5 /* 6 */,
    128 + -4 /* 7 */,
    128 + -3 /* 8 */,
    128 + -2 /* 9 */,
    128 + -1 /* 10 */,
    128 + 0  /* 11 */,
    128 + 0  /* 12 */,
    128 + 0  /* 13 */,
    128 + 0  /* 14 */,
    128 + 0  /* 15 */,
    128 + 1  /* 16 */,
    128 + 2  /* 17 */,
    128 + 3  /* 18/00-18/09 */,
    128 + 4  /* 18/10-18/19 */,
    128 + 5  /* 18/20-18/29 */,
    128 + 6  /* 18/30-18/39 */,
    128 + 7  /* 18/40-18/49 */,
    128 + 8  /* 18/50-18/59 */,
    128 + 9  /* 18/60-18/69 */,
    128 + 10 /* 18/70-18/79 */,
    128 + 11 /* 18/80-18/89 */,
    128 + 12 /* 18/90-18/99 */,
    128 + 13 /* 18/100-18/109 */,
    128 + 14 /* 18/110-18/119 */,
    128 + 15 /* 18/120-18/129 */,
    128 + 16 /* 18/130-18/139 */,
    128 + 17 /* 18/140-18/149 */,
    128 + 18 /* 18/150-18/159 */,
    128 + 19 /* 18/160-18/169 */,
    128 + 20 /* 18/170-18/179 */,
    128 + 21 /* 18/180-18/189 */,
    128 + 22 /* 18/190-18/199 */,
    128 + 23 /* 18/200-18/209 */,
    128 + 24 /* 18/210-18/219 */,
    128 + 25 /* 18/220-18/239 */,
    128 + 26 /* 18/230-18/249 */,
    128 + 26 /* 18/240-18/249 */,
    128 + 27 /* 18/250+ */
};


/*
 * Stat Table (INT) -- bonus to hit (plus 128)
 */
const byte adj_int_th[] =
{
	128 + -3	/* 3 */,
	128 + -2	/* 4 */,
	128 + -1	/* 5 */,
	128 + -1	/* 6 */,
	128 + 0	/* 7 */,
	128 + 0	/* 8 */,
	128 + 0	/* 9 */,
	128 + 0	/* 10 */,
	128 + 0	/* 11 */,
	128 + 0	/* 12 */,
	128 + 0	/* 13 */,
	128 + 0	/* 14 */,
	128 + 0	/* 15 */,
	128 + 0	/* 16 */,
	128 + 0	/* 17 */,
	128 + 1	/* 18/00-18/09 */,
	128 + 1	/* 18/10-18/19 */,
	128 + 1	/* 18/20-18/29 */,
	128 + 1	/* 18/30-18/39 */,
	128 + 1	/* 18/40-18/49 */,
	128 + 2	/* 18/50-18/59 */,
	128 + 2	/* 18/60-18/69 */,
	128 + 3	/* 18/70-18/79 */,
	128 + 3	/* 18/80-18/89 */,
	128 + 4	/* 18/90-18/99 */,
	128 + 5	/* 18/100-18/109 */,
	128 + 6	/* 18/110-18/119 */,
	128 + 7	/* 18/120-18/129 */,
	128 + 8	/* 18/130-18/139 */,
	128 + 9	/* 18/140-18/149 */,
	128 + 10	/* 18/150-18/159 */,
	128 + 11	/* 18/160-18/169 */,
	128 + 12	/* 18/170-18/179 */,
	128 + 13	/* 18/180-18/189 */,
	128 + 14	/* 18/190-18/199 */,
	128 + 15	/* 18/200-18/209 */,
	128 + 16	/* 18/210-18/219 */,
	128 + 17	/* 18/220-18/229 */,
	128 + 18	/* 18/230-18/239 */,
	128 + 18	/* 18/240-18/249 */,
	128 + 19	/* 18/250+ */
};


/*
 * Stat Table (STR) -- weight limit in deca-pounds
 */
const byte adj_str_wgt[] =
{
	5	/* 3 */,
	6	/* 4 */,
	7	/* 5 */,
	8	/* 6 */,
	9	/* 7 */,
	10	/* 8 */,
	11	/* 9 */,
	12	/* 10 */,
	13	/* 11 */,
	14	/* 12 */,
	15	/* 13 */,
	16	/* 14 */,
	17	/* 15 */,
	18	/* 16 */,
	19	/* 17 */,
	20	/* 18/00-18/09 */,
	22	/* 18/10-18/19 */,
	24	/* 18/20-18/29 */,
	26	/* 18/30-18/39 */,
	28	/* 18/40-18/49 */,
	30	/* 18/50-18/59 */,
	30	/* 18/60-18/69 */,
	30	/* 18/70-18/79 */,
	30	/* 18/80-18/89 */,
	30	/* 18/90-18/99 */,
	30	/* 18/100-18/109 */,
	30	/* 18/110-18/119 */,
	30	/* 18/120-18/129 */,
	30	/* 18/130-18/139 */,
	30	/* 18/140-18/149 */,
	30	/* 18/150-18/159 */,
	30	/* 18/160-18/169 */,
	30	/* 18/170-18/179 */,
	30	/* 18/180-18/189 */,
	30	/* 18/190-18/199 */,
	30	/* 18/200-18/209 */,
	30	/* 18/210-18/219 */,
	30	/* 18/220-18/229 */,
	30	/* 18/230-18/239 */,
	30	/* 18/240-18/249 */,
	30	/* 18/250+ */
};


/*
 * Stat Table (STR) -- weapon weight limit in pounds
 */
const byte adj_str_hold[] =
{
	4	/* 3 */,
	5	/* 4 */,
	6	/* 5 */,
	7	/* 6 */,
	8	/* 7 */,
	10	/* 8 */,
	12	/* 9 */,
	14	/* 10 */,
	16	/* 11 */,
	18	/* 12 */,
	20	/* 13 */,
	22	/* 14 */,
	24	/* 15 */,
	26	/* 16 */,
	28	/* 17 */,
	30	/* 18/00-18/09 */,
	30	/* 18/10-18/19 */,
	35	/* 18/20-18/29 */,
	40	/* 18/30-18/39 */,
	45	/* 18/40-18/49 */,
	50	/* 18/50-18/59 */,
	55	/* 18/60-18/69 */,
	60	/* 18/70-18/79 */,
	65	/* 18/80-18/89 */,
	70	/* 18/90-18/99 */,
	80	/* 18/100-18/109 */,
	80	/* 18/110-18/119 */,
	80	/* 18/120-18/129 */,
	80	/* 18/130-18/139 */,
	80	/* 18/140-18/149 */,
	90	/* 18/150-18/159 */,
	90	/* 18/160-18/169 */,
	90	/* 18/170-18/179 */,
	90	/* 18/180-18/189 */,
	90	/* 18/190-18/199 */,
	100	/* 18/200-18/209 */,
	100	/* 18/210-18/219 */,
	100	/* 18/220-18/229 */,
	100	/* 18/230-18/239 */,
	100	/* 18/240-18/249 */,
	100	/* 18/250+ */
};


/*
 * Stat Table (STR) -- digging value
 */
const byte adj_str_dig[] =
{
	0	/* 3 */,
	0	/* 4 */,
	1	/* 5 */,
	2	/* 6 */,
	3	/* 7 */,
	4	/* 8 */,
	4	/* 9 */,
	5	/* 10 */,
	5	/* 11 */,
	6	/* 12 */,
	6	/* 13 */,
	7	/* 14 */,
	7	/* 15 */,
	8	/* 16 */,
	8	/* 17 */,
	9	/* 18/00-18/09 */,
	10	/* 18/10-18/19 */,
	12	/* 18/20-18/29 */,
	15	/* 18/30-18/39 */,
	20	/* 18/40-18/49 */,
	25	/* 18/50-18/59 */,
	30	/* 18/60-18/69 */,
	35	/* 18/70-18/79 */,
	40	/* 18/80-18/89 */,
	45	/* 18/90-18/99 */,
	50	/* 18/100-18/109 */,
	55	/* 18/110-18/119 */,
	60	/* 18/120-18/129 */,
	65	/* 18/130-18/139 */,
	70	/* 18/140-18/149 */,
	75	/* 18/150-18/159 */,
	80	/* 18/160-18/169 */,
	85	/* 18/170-18/179 */,
	90	/* 18/180-18/189 */,
	95	/* 18/190-18/199 */,
	100	/* 18/200-18/209 */,
	105	/* 18/210-18/219 */,
	110	/* 18/220-18/229 */,
	113	/* 18/230-18/239 */,
	115	/* 18/240-18/249 */,
	117	/* 18/250+ */
};


/*
 * Stat Table (STR) -- help index into the "blow" table
 */
const byte adj_str_blow[] =
{
	3	/* 3 */,
	4	/* 4 */,
	5	/* 5 */,
	6	/* 6 */,
	7	/* 7 */,
	8	/* 8 */,
	9	/* 9 */,
	10	/* 10 */,
	11	/* 11 */,
	12	/* 12 */,
	13	/* 13 */,
	14	/* 14 */,
	15	/* 15 */,
	16	/* 16 */,
	17	/* 17 */,
	20 /* 18/00-18/09 */,
	30 /* 18/10-18/19 */,
	40 /* 18/20-18/29 */,
	50 /* 18/30-18/39 */,
	60 /* 18/40-18/49 */,
	70 /* 18/50-18/59 */,
	80 /* 18/60-18/69 */,
	90 /* 18/70-18/79 */,
	100 /* 18/80-18/89 */,
	110 /* 18/90-18/99 */,
	120 /* 18/100-18/109 */,
	130 /* 18/110-18/119 */,
	140 /* 18/120-18/129 */,
	150 /* 18/130-18/139 */,
	160 /* 18/140-18/149 */,
	170 /* 18/150-18/159 */,
	180 /* 18/160-18/169 */,
	190 /* 18/170-18/179 */,
	200 /* 18/180-18/189 */,
	210 /* 18/190-18/199 */,
	220 /* 18/200-18/209 */,
	230 /* 18/210-18/219 */,
	240 /* 18/220-18/229 */,
	245 /* 18/230-18/239 */,
	250 /* 18/240-18/249 */,
	255 /* 18/250+ */
};



/*
 * Stat Table (DEX) -- index into the "blow" table
 */
const byte adj_dex_blow[] =
{
	0	/* 3 */,
	0	/* 4 */,
	0	/* 5 */,
	0	/* 6 */,
	0	/* 7 */,
	0	/* 8 */,
	0	/* 9 */,
	1	/* 10 */,
	1	/* 11 */,
	1	/* 12 */,
	1	/* 13 */,
	1	/* 14 */,
	1	/* 15 */,
	1	/* 16 */,
	1	/* 17 */,
	1	/* 18/00-18/09 */,
	2	/* 18/10-18/19 */,
	2	/* 18/20-18/29 */,
	2	/* 18/30-18/39 */,
	2	/* 18/40-18/49 */,
	3	/* 18/50-18/59 */,
	3	/* 18/60-18/69 */,
	4	/* 18/70-18/79 */,
	4	/* 18/80-18/89 */,
	5	/* 18/90-18/99 */,
	6	/* 18/100-18/109 */,
	7	/* 18/110-18/119 */,
	7	/* 18/120-18/129 */,
    8	/* 18/130-18/139 */,
	8	/* 18/140-18/149 */,
	9	/* 18/150-18/159 */,
	9	/* 18/160-18/169 */,
	10	/* 18/170-18/179 */,
	10	/* 18/180-18/189 */,
	10	/* 18/190-18/199 */,
	10	/* 18/200-18/209 */,
	10	/* 18/210-18/219 */,
	11  /* 18/220-18/229 */,
	11  /* 18/230-18/239 */,
	11  /* 18/240-18/249 */,
	11  /* 18/250+ */
};


/*
 * Stat Table (AGI) -- modifier to speed
 */
const byte adj_agi_speed[] =
{
	128 + -5	/* 3 */,
	128 + -4	/* 4 */,
	128 + -3	/* 5 */,
	128 + -3	/* 6 */,
	128 + -2	/* 7 */,
	128 + -2	/* 8 */,
	128 + -1	/* 9 */,
	128 + -1	/* 10 */,
	128 + 0		/* 11 */,
	128 + 0		/* 12 */,
	128 + 0		/* 13 */,
	128 + 0		/* 14 */,
	128 + 1		/* 15 */,
	128 + 1		/* 16 */,
	128 + 2		/* 17 */,
	128 + 2		/* 18/00-18/09 */,
	128 + 3		/* 18/10-18/19 */,
	128 + 3		/* 18/20-18/29 */,
	128 + 4		/* 18/30-18/39 */,
	128 + 4		/* 18/40-18/49 */,
	128 + 5	/* 18/50-18/59 */,
	128 + 5	/* 18/60-18/69 */,
	128 + 6	/* 18/70-18/79 */,
	128 + 6	/* 18/80-18/89 */,
	128 + 7	/* 18/90-18/99 */,
	128 + 7	/* 18/100-18/109 */,
	128 + 8	/* 18/110-18/119 */,
	128 + 8	/* 18/120-18/129 */,
	128 + 9	/* 18/130-18/139 */,
	128 + 9	/* 18/140-18/149 */,
	128 + 10	/* 18/150-18/159 */,
	128 + 10	/* 18/160-18/169 */,
	128 + 11	/* 18/170-18/179 */,
	128 + 11	/* 18/180-18/189 */,
	128 + 12	/* 18/190-18/199 */,
	128 + 13	/* 18/200-18/209 */,
	128 + 14	/* 18/210-18/219 */,
	128 + 15	/* 18/220-18/229 */,
	128 + 16	/* 18/230-18/239 */,
	128 + 17	/* 18/240-18/249 */,
	128 + 18	/* 18/250+ */
};


/*
 * Stat Table (AGI) -- chance of avoiding "theft" and "falling"
 */
const byte adj_agi_safe[] =
{
	0	/* 3 */,
	1	/* 4 */,
	2	/* 5 */,
	3	/* 6 */,
	4	/* 7 */,
	5	/* 8 */,
	5	/* 9 */,
	6	/* 10 */,
	6	/* 11 */,
	7	/* 12 */,
	7	/* 13 */,
	8	/* 14 */,
	8	/* 15 */,
	9	/* 16 */,
	9	/* 17 */,
	10	/* 18/00-18/09 */,
	11	/* 18/10-18/19 */,
	13	/* 18/20-18/29 */,
	16	/* 18/30-18/39 */,
	20	/* 18/40-18/49 */,
	25	/* 18/50-18/59 */,
	30	/* 18/60-18/69 */,
	35	/* 18/70-18/79 */,
	40	/* 18/80-18/89 */,
	45	/* 18/90-18/99 */,
	50	/* 18/100-18/109 */,
	60	/* 18/110-18/119 */,
	70	/* 18/120-18/129 */,
	80	/* 18/130-18/139 */,
	90	/* 18/140-18/149 */,
	100	/* 18/150-18/159 */,
	100	/* 18/160-18/169 */,
	100	/* 18/170-18/179 */,
	100	/* 18/180-18/189 */,
	100	/* 18/190-18/199 */,
	100	/* 18/200-18/209 */,
	100	/* 18/210-18/219 */,
	100	/* 18/220-18/229 */,
	100	/* 18/230-18/239 */,
	100	/* 18/240-18/249 */,
	100	/* 18/250+ */
};


/*
 * Stat Table (CON) -- base regeneration rate
 */
const byte adj_con_fix[] =
{
	0	/* 3 */,
	0	/* 4 */,
	0	/* 5 */,
	0	/* 6 */,
	0	/* 7 */,
	1	/* 8 */,
	1	/* 9 */,
	1	/* 10 */,
	1	/* 11 */,
	1	/* 12 */,
	2	/* 13 */,
	2	/* 14 */,
	2	/* 15 */,
	2	/* 16 */,
	2	/* 17 */,
	3	/* 18/00-18/09 */,
	3	/* 18/10-18/19 */,
	3	/* 18/20-18/29 */,
	4	/* 18/30-18/39 */,
	4	/* 18/40-18/49 */,
	4	/* 18/50-18/59 */,
	4	/* 18/60-18/69 */,
	5	/* 18/70-18/79 */,
	5	/* 18/80-18/89 */,
	5	/* 18/90-18/99 */,
	6	/* 18/100-18/109 */,
	6	/* 18/110-18/119 */,
	6	/* 18/120-18/129 */,
	7	/* 18/130-18/139 */,
	7	/* 18/140-18/149 */,
	7	/* 18/150-18/159 */,
	8	/* 18/160-18/169 */,
	8	/* 18/170-18/179 */,
	8	/* 18/180-18/189 */,
	9	/* 18/190-18/199 */,
	9	/* 18/200-18/209 */,
	9	/* 18/210-18/219 */,
	10	/* 18/220-18/229 */,
	10	/* 18/230-18/239 */,
	10	/* 18/240-18/249 */,
	11	/* 18/250+ */
};


/*
 * Stat Table (CON) -- extra hit die points above SIZ (plus 128)
 */
const byte adj_con_die[] =
{
	128 + -3	/* 3 */,
	128 + -2	/* 4 */,
	128 + -2	/* 5 */,
	128 + -1	/* 6 */,
	128 + -1	/* 7 */,
	128 + 0	/* 8 */,
	128 + 0	/* 9 */,
	128 + 0	/* 10 */,
	128 + 0	/* 11 */,
	128 + 0	/* 12 */,
	128 + 1	/* 13 */,
	128 + 1	/* 14 */,
	128 + 2	/* 15 */,
	128 + 2	/* 16 */,
	128 + 2	/* 17 */,
	128 + 3	/* 18/00-18/09 */,
	128 + 3	/* 18/10-18/19 */,
	128 + 4	/* 18/20-18/29 */,
	128 + 4	/* 18/30-18/39 */,
	128 + 4	/* 18/40-18/49 */,
	128 + 5	/* 18/50-18/59 */,
	128 + 5	/* 18/60-18/69 */,
	128 + 6	/* 18/70-18/79 */,
	128 + 6	/* 18/80-18/89 */,
	128 + 7	/* 18/90-18/99 */,
	128 + 7	/* 18/100-18/109 */,
	128 + 8	/* 18/110-18/119 */,
	128 + 9	/* 18/120-18/129 */,
	128 + 10	/* 18/130-18/139 */,
	128 + 11 	/* 18/140-18/149 */,
	128 + 12	/* 18/150-18/159 */,
	128 + 13	/* 18/160-18/169 */,
	128 + 14	/* 18/170-18/179 */,
	128 + 15	/* 18/180-18/189 */,
	128 + 16	/* 18/190-18/199 */,
	128 + 17	/* 18/200-18/209 */,
	128 + 18	/* 18/210-18/219 */,
	128 + 19	/* 18/220-18/229 */,
	128 + 20	/* 18/230-18/239 */,
	128 + 21	/* 18/240-18/249 */,
	128 + 22	/* 18/250+ */
};


/*
 * Stat Table (CON) -- reserve mana for spells.
 */
const byte adj_con_reserve[] =
{
	0	/* 3 */,
	1	/* 4 */,
	2	/* 5 */,
	3	/* 6 */,
	4	/* 7 */,
	5	/* 8 */,
	6	/* 9 */,
	7	/* 10 */,
	8	/* 11 */,
	9	/* 12 */,
	10	/* 13 */,
	11	/* 14 */,
	12	/* 15 */,
	13	/* 16 */,
	14	/* 17 */,
	15 /* 18/00-18/09 */,
	17 /* 18/10-18/19 */,
	19 /* 18/20-18/29 */,
	21 /* 18/30-18/39 */,
	24 /* 18/40-18/49 */,
	27 /* 18/50-18/59 */,
	30 /* 18/60-18/69 */,
	34 /* 18/70-18/79 */,
	38 /* 18/80-18/89 */,
	42 /* 18/90-18/99 */,
	46 /* 18/100-18/109 */,
	50 /* 18/110-18/119 */,
	55 /* 18/120-18/129 */,
	60 /* 18/130-18/139 */,
	65 /* 18/140-18/149 */,
	70 /* 18/150-18/159 */,
	75 /* 18/160-18/169 */,
	80 /* 18/170-18/179 */,
	85 /* 18/180-18/189 */,
	90 /* 18/190-18/199 */,
	95 /* 18/200-18/209 */,
	100 /* 18/210-18/219 */,
	105 /* 18/220-18/229 */,
	110 /* 18/230-18/239 */,
	115 /* 18/240-18/249 */,
	120 /* 18/250+ */
};


/*
 * Stat Table (SIZ) -- extra hit die points on top of standard 1d10 (plus 128)
 * This is the total bonus at 1st level, plus every 2 levels thereafter.
 */
const byte adj_siz_die[] =
{
	128 + -6	/* 3 */,
	128 + -5	/* 4 */,
	128 + -4	/* 5 */,
	128 + -3	/* 6 */,
	128 + -2	/* 7 */,
	128 + -1	/* 8 */,
	128 + 0	/* 9 */,
	128 + 0	/* 10 */,
	128 + 1	/* 11 */,
	128 + 2	/* 12 */,
	128 + 2	/* 13 */,
	128 + 3	/* 14 */,
	128 + 3	/* 15 */,
	128 + 4	/* 16 */,
	128 + 4	/* 17 */,
	128 + 5	/* 18/00-18/09 */,
	128 + 6	/* 18/10-18/19 */,
	128 + 7	/* 18/20-18/29 */,
	128 + 8	/* 18/30-18/39 */,
	128 + 9	/* 18/40-18/49 */,
	128 + 10	/* 18/50-18/59 */,
	128 + 11	/* 18/60-18/69 */,
	128 + 12	/* 18/70-18/79 */,
	128 + 13	/* 18/80-18/89 */,
	128 + 14	/* 18/90-18/99 */,
	128 + 15	/* 18/100-18/109 */,
	128 + 16	/* 18/110-18/119 */,
	128 + 18	/* 18/120-18/129 */,
	128 + 20	/* 18/130-18/139 */,
	128 + 22 	/* 18/140-18/149 */,
	128 + 24	/* 18/150-18/159 */,
	128 + 26	/* 18/160-18/169 */,
	128 + 28	/* 18/170-18/179 */,
	128 + 30	/* 18/180-18/189 */,
	128 + 32	/* 18/190-18/199 */,
	128 + 34	/* 18/200-18/209 */,
	128 + 36	/* 18/210-18/219 */,
	128 + 38	/* 18/220-18/229 */,
	128 + 40	/* 18/230-18/239 */,
	128 + 42	/* 18/240-18/249 */,
	128 + 45	/* 18/250+ */
};


/*
 * Stat Table (SIZ) -- num_charge limit
 */
const byte adj_charge_siz[] =
{
	0 	/* 3 */,
	0 	/* 4 */,
	0 	/* 5 */,
	0 	/* 6 */,
	0 	/* 7 */,
	1 	/* 8 */,
	1 	/* 9 */,
	1 	/* 10 */,
	1 	/* 11 */,
	1 	/* 12 */,
	2 	/* 13 */,
	2 	/* 14 */,
	2 	/* 15 */,
	2 	/* 16 */,
	2 	/* 17 */,
	3 	/* 18/00-18/09 */,
	3 	/* 18/10-18/19 */,
	3 	/* 18/20-18/29 */,
	4 	/* 18/30-18/39 */,
	4 	/* 18/40-18/49 */,
	5 	/* 18/50-18/59 */,
	5 	/* 18/60-18/69 */,
	6 	/* 18/70-18/79 */,
	6 	/* 18/80-18/89 */,
	6 	/* 18/90-18/99 */,
	7 	/* 18/100-18/109 */,
	7 	/* 18/110-18/119 */,
	7	/* 18/120-18/129 */,
	8	/* 18/130-18/139 */,
	8	/* 18/140-18/149 */,
	9	/* 18/150-18/159 */,
	9	/* 18/160-18/169 */,
	10	/* 18/170-18/179 */,
	10	/* 18/180-18/189 */,
	11	/* 18/190-18/199 */,
	12	/* 18/200-18/209 */,
	13	/* 18/210-18/219 */,
	15	/* 18/220-18/229 */,
	17	/* 18/230-18/239 */,
	19	/* 18/240-18/249 */,
	22	/* 18/250+ */
};


/*
 * This table is used to help calculate the number of blows the player can
 * make in a single round of attacks (one player turn) with a normal weapon.
 *
 * This number ranges from a single blow/round for weak players to up to six
 * blows/round for powerful warriors.
 *
 * Note that certain artifacts and ego-items give "bonus" blows/round.
 *
 * First, from the player class, we extract some values:
 *
 *    Warrior --> num = 6; mul = 5; div = MAX(30, weapon_weight);
 *    Mage    --> num = 4; mul = 2; div = MAX(40, weapon_weight);
 *    Priest  --> num = 5; mul = 3; div = MAX(35, weapon_weight);
 *    Rogue   --> num = 5; mul = 3; div = MAX(30, weapon_weight);
 *    Ranger  --> num = 5; mul = 4; div = MAX(35, weapon_weight);
 *    Paladin --> num = 5; mul = 4; div = MAX(30, weapon_weight);
 *
 * New classes:
 *    Thief   --> num = 5; mul = 4; div = MAX(30, weapon_weight);
 *    Archer  --> num = 5; mul = 3; div = MAX(30, weapon_weight);
 * To get "P", we look up the relevant "adj_str_blow[]" (see above),
 * multiply it by "mul", and then divide it by "div", rounding down.
 *
 * To get "D", we look up the relevant "adj_dex_blow[]" (see above),
 * note especially column 6 (DEX 18/101) and 11 (DEX 18/150).
 *
 * The player gets "blows_table[P][D]" blows/round, as shown below,
 * up to a maximum of "num" blows/round, plus any "bonus" blows/round.
 */

const byte blows_table[12][12] =
{
	/* P/D */
	/* 0,   1,   2,   3,   4,   5,   6,   7,   8,   9,  10,  11+ */

	/* 0  */
	{  1,   1,   1,   1,   1,   1,   2,   2,   2,   2,   2,   3 },

	/* 1  */
	{  1,   1,   1,   1,   2,   2,   3,   3,   3,   4,   4,   4 },

	/* 2  */
	{  1,   1,   2,   2,   3,   3,   4,   4,   4,   5,   5,   5 },

	/* 3  */
	{  1,   2,   2,   3,   3,   4,   4,   4,   5,   5,   5,   5 },

	/* 4  */
	{  1,   2,   2,   3,   3,   4,   4,   5,   5,   5,   5,   5 },

	/* 5  */
	{  2,   2,   3,   3,   4,   4,   5,   5,   5,   5,   5,   6 },

	/* 6  */
	{  2,   2,   3,   3,   4,   4,   5,   5,   5,   5,   5,   6 },

	/* 7  */
	{  2,   3,   3,   4,   4,   4,   5,   5,   5,   5,   5,   6 },

	/* 8  */
	{  3,   3,   3,   4,   4,   4,   5,   5,   5,   5,   6,   6 },

	/* 9  */
	{  3,   3,   4,   4,   4,   4,   5,   5,   5,   5,   6,   6 },

	/* 10 */
	{  3,   3,   4,   4,   4,   4,   5,   5,   5,   6,   6,   6 },

	/* 11+ */
	{  3,   3,   4,   4,   4,   4,   5,   5,   6,   6,   6,   6 },
};



#if 0

/*
 * This is the "old" table used to calculate multiple blows.
 *
 * Note that this table used a different indexing scheme to determine "P"
 */

byte old_blows_table[11][12] =
{
	/* P/D */
	/* 3,  10, /01, /50, /90,/100,/101,/110,/120,/130,/140,/150 */

	/* 0+ */
	{  1,   1,   1,   1,   1,   1,   2,   2,   2,   2,   2,   3},

	/* 2+ */
	{  1,   1,   1,   1,   2,   2,   3,   3,   3,   3,   3,   4},

	/* 3+ */
	{  1,   1,   1,   2,   2,   3,   4,   4,   4,   4,   4,   5},

	/* 4+ */
	{  1,   1,   2,   2,   3,   3,   4,   4,   4,   5,   5,   5},

	/* 6+ */
	{  1,   2,   2,   3,   3,   4,   4,   4,   5,   5,   5,   5},

	/* 8+ */
	{  1,   2,   2,   3,   4,   4,   4,   5,   5,   5,   5,   5},

	/* 10+ */
	{  2,   2,   3,   3,   4,   4,   5,   5,   5,   5,   5,   6},

	/* 13+ */
	{  2,   3,   3,   3,   4,   4,   5,   5,   5,   5,   5,   6},

	/* 15+ */
	{  3,   3,   3,   4,   4,   4,   5,   5,   5,   5,   6,   6},

	/* 18+ */
	{  3,   3,   3,   4,   4,   4,   5,   5,   5,   5,   6,   6},

	/* 20+ */
	{  3,   3,   4,   4,   4,   4,   5,   5,   5,   6,   6,   6}
};

#endif


/*
 * This table allows quick conversion from "speed" to "energy"
 * The basic function WAS ((S>=110) ? (S-110) : (100 / (120-S)))
 * Note that table access is *much* quicker than computation.
 *
 * Note that the table has been changed at high speeds.  From
 * "Slow (-40)" to "Fast (+30)" is pretty much unchanged, but
 * at speeds above "Fast (+30)", one approaches an asymptotic
 * effective limit of 50 energy per turn.  This means that it
 * is relatively easy to reach "Fast (+30)" and get about 40
 * energy per turn, but then speed becomes very "expensive",
 * and you must get all the way to "Fast (+50)" to reach the
 * point of getting 45 energy per turn.  After that point,
 * furthur increases in speed are more or less pointless,
 * except to balance out heavy inventory.
 *
 * Note that currently the fastest monster is "Fast (+30)".
 *
 * It should be possible to lower the energy threshhold from
 * 100 units to 50 units, though this may interact badly with
 * the (compiled out) small random energy boost code.  It may
 * also tend to cause more "clumping" at high speeds.
 */
const byte extract_energy[200] =
{
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* Slow */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* S-50 */     1,  1,  1,  1,  1,  1,  1,  1,  1,  1,
	/* S-40 */     2,  2,  2,  2,  2,  2,  2,  2,  2,  2,
	/* S-30 */     2,  2,  2,  2,  2,  2,  2,  3,  3,  3,
	/* S-20 */     3,  3,  3,  4,  4,  4,  4,  5,  5,  5,
	/* S-10 */     5,  6,  6,  7,  7,  8,  8,  9,  9,  9,
	/* Norm */    10, 11, 12, 13, 14, 15, 16, 17, 18, 19,
	/* F+10 */    20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
	/* F+20 */    30, 31, 32, 33, 34, 35, 36, 36, 37, 37,
	/* F+30 */    38, 38, 39, 39, 40, 40, 40, 41, 41, 41,
	/* F+40 */    42, 42, 42, 43, 43, 43, 44, 44, 44, 44,
	/* F+50 */    45, 45, 45, 45, 45, 46, 46, 46, 46, 46,
	/* F+60 */    47, 47, 47, 47, 47, 48, 48, 48, 48, 48,
	/* F+70 */    49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
	/* Fast */    49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
};







/*
 * Base experience levels, may be adjusted up for race and/or class
 */
const s32b player_exp[PY_MAX_LEVEL] =
{
	10,
	25,
	45,
	70,
	100,
	140,
	200,
	280,
	380,
	500,
	650,
	850,
	1100,
	1400,
	1800,
	2300,
	2900,
	3600,
	4400,
	5400,
	6800,
	8400,
	10200,
	12500,
	17500,
	25000,
	35000L,
	50000L,
	75000L,
	100000L,
	150000L,
	200000L,
	275000L,
	350000L,
	450000L,
	550000L,
	700000L,
	850000L,
	1000000L,
	1250000L,
	1500000L,
	1800000L,
	2100000L,
	2400000L,
	2700000L,
	3000000L,
	3500000L,
	4000000L,
	4500000L,
	5000000L
};


/*
 * Player Sexes
 *
 *	Title,
 *	Winner
 */
const player_sex sex_info[MAX_SEXES] =
{
	{
		"Female",
		"Queen"
	},

	{
		"Male",
		"King"
	}
};


/*
 * Abbreviations of healthy stats
 */
const cptr stat_names[A_MAX] =
{
	"STR: ", "INT: ", "WIS: ", "DEX: ", "CON: ", "CHR: ", "AGI: ", "SIZ: "
};

/*
 * Abbreviations of damaged stats
 */
const cptr stat_names_reduced[A_MAX] =
{
	"Str: ", "Int: ", "Wis: ", "Dex: ", "Con: ", "Chr: ", "Agi: ", "Siz: "
};

/*
 * Abbreviations of damaged stats -- no sidebar
 */
const cptr stat_names_reduced_short[A_MAX] =
{
	"Str", "Int", "Wis", "Dex", "Con", "Chr", "Agi", "Siz"
};

/*
 * Certain "screens" always use the main screen, including News, Birth,
 * Dungeon, Tomb-stone, High-scores, Macros, Colors, Visuals, Options.
 *
 * Later, special flags may allow sub-windows to "steal" stuff from the
 * main window, including File dump (help), File dump (artifacts, uniques),
 * Character screen, Small scale map, Previous Messages, Store screen, etc.
 */
const cptr window_flag_desc[32] =
{
	"Display inven/equip",
	"Display equip/inven",
	"Display player (basic)",
	"Display player (extra)",
	"Display player (compact)",
	"Display player (status)",
	"Display messages",
	"Display dungeon map",
	"Display overhead view",
	"Display monster recall",
	"Display object recall",
	"Display feature recall (not used)",
	"Display room description",
	"Display snap-shot (not used)",
	"Display visible monsters",
	"Display visible objects",
	"Display help (not used)",
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL,
	NULL
};

const cptr inscrip_text[MAX_INSCRIP] =
{
	NULL,
	"terrible",
	"worthless",
	"cursed",
	"broken",
	"empty",
	"good",
	"excellent",
	"special",
	"not cursed",
	"very good",
	"great",
	"superb",
	"unbreakable",
	"ungettable",
	"nonmagical",
	"magical",
	"magic item",
	"ego item",
	"high ego item",
	"artifact",
	"unruned",
	"runed",
	"average",
	"valuable",
	"unusual",
	"coated",
	"sustain",
	"high resist",
	"enchanted",
	"low resist",
	"ignore",
	"blessed",
	"slaying",
	"branded",
	"poisoned",
	"light source",
	"acidproof",
	"fireproof",
	"waterproof",
	"theftproof",
	"vampiric"
};

const int object_xtra_what[OBJECT_XTRA_MAX_HIDDEN] =
{
	0,
	2,
	2,
	3,
	2,
	2,
	3,
	1,
	1,
	1,
	3,
	2,
	2,
	2,
	2,
	4,
	1,
	2,
	3,
	4
};

const u32b object_xtra_base[OBJECT_XTRA_MAX_HIDDEN] =
{
	0,
	TR2_SUST_STR,
	TR2_RES_POIS,
	TR3_SLOW_DIGEST,
	TR2_RES_ACID,
	TR2_IGNORE_ACID,
	TR3_BLESSED,
	TR1_SLAY_NATURAL,
	TR1_BRAND_ACID,
	TR1_BRAND_POIS,
	TR3_LITE,
	TR2_IGNORE_ACID,
	TR2_IGNORE_FIRE,
	TR2_IGNORE_WATER,
	TR2_IGNORE_THEFT,
	TR4_HURT_LITE,
	1L,	/* Magic item - flag 1 */
	1L,	/* Magic item - flag 2 */
	1L,	/* Magic item - flag 3 */
	1L	/* Magic item - flag 4 */
};


const int object_xtra_size[OBJECT_XTRA_MAX_HIDDEN] =
{
	0,
	6,
	12,
	16,
	4,
	5,
	1,
	8,
	4,
	1,
	1,
	1,
	1,
	1,
	1,
	4,
	32,	/* Magic item - flag 1 */
	32,	/* Magic item - flag 2 */
	32,	/* Magic item - flag 3 */
	32	/* Magic item - flag 4 */
};


/*
 * Description of each object group.
 */
const object_grouper object_group[] =
{
	{"Sword",	TV_SWORD},
	{"Polearm",	TV_POLEARM},
	{"Hafted Weapon",	TV_HAFTED},
	{"Bow",		TV_BOW},
	{"Arrow",	TV_ARROW},
	{"Bolt"	,	TV_BOLT},
	{"Shot"	,	TV_SHOT},
	{"Shield",	TV_SHIELD},
	{"Crown"	,	TV_CROWN},
	{"Helm",	TV_HELM},
	{"Gloves",	TV_GLOVES},
	{"Boots"	,	TV_BOOTS},
	{"Cloak"	,	TV_CLOAK},
	{"Dragon Scale Mail",	TV_DRAG_ARMOR},
	{"Hard Armor",	TV_HARD_ARMOR},
	{"Soft Armor",	TV_SOFT_ARMOR},
	{"Ring",		TV_RING},
	{"Amulet",	TV_AMULET},
	{"Lite",	TV_LITE},
	{"Potion"	 ,	TV_POTION},
	{"Scroll"	 ,	TV_SCROLL},
	{"Wand",	TV_WAND},
	{"Staff"	,	TV_STAFF},
	{"Rod"	,	TV_ROD},
	{"Prayer Book",	TV_PRAYER_BOOK},
	{"Magic Book",	TV_MAGIC_BOOK},
	{"Song Book"	,	TV_SONG_BOOK},
	{"Instrument",	TV_INSTRUMENT},
	{"Runestone"	,	TV_RUNESTONE},
	{"Map"		,	TV_MAP},
	{"Spike"	,	TV_SPIKE},
	{"Rope" ,	TV_ROPE},
	{"Digger"	 ,	TV_DIGGING},
	{"Mushroom"	,	TV_MUSHROOM},
	{"Flask"	,	TV_FLASK},
	{"Food"		,	TV_FOOD},
	{"Service", 	TV_SERVICE},
	{"Magic Bag"	,	TV_BAG},
	{"Container"	,	TV_HOLD},
	{"Statue"	 ,	TV_STATUE},
	{"Assembly"	 ,	TV_ASSEMBLY},
	{"Skeleton"	,	TV_BONE},
	{"Corpse"	 ,	TV_BODY},
	{"Egg"		 ,	TV_EGG},
	{"Skin"		 ,	TV_SKIN},
	{"Junk",	TV_JUNK},
	{"Gold",	TV_GOLD},
	{"Gemstones",	TV_GEMS},
	{"Blessed Weapon", TV_BLESSED},
	{NULL, 0}
};

const cptr magic_name[4][32] =
{
	/* TR1_ */
	{
		"of Strength and Size",
		"of Intelligence",
		"of Wisdom",
		"of Dexterity and Agility",
		"of Constitution",
		"of Charisma",
		"of Spell Resistance",
		"of Device Mastery",
		"of Stealth",
		"of Searching",
		"of Infravision",
		"of Tunnelling",
		"of Speed",
		"of Extra Attacks",
		"of Extra Shots",
		"of Extra Might",
		"of Slay Nature",
		"of Slay Evil",
		"of Slay Undead",
		"of Slay Demon",
		"of Slay Orc",
		"of Slay Troll",
		"of Slay Giant",
		"of Slay Dragon",
		"of Execute Dragon",
		"of Execute Demon",
		"of Execute Undead",
		"of Poison",
		"of Acid",
		"of Electricity",
		"of Fire",
		"of Cold"
	},

	/* TR2_ */
	{
		"of Sustain Strength and Size",
		"of Sustain Intelligence",
		"of Sustain Wisdom",
		"of Sustain Dexterity and Agility",
		"of Sustain Constitution",
		"of Sustain Charisma",
		"(Acid proof)",
		"(Lightning proof)",
		"(Fire proof)",
		"(Frost proof)",
		"(Water proof)",
		"(Theft proof)",
		"of Acid Immunity",
		"of Lightening Immunity",
		"of Fire Immunity",
		"of Frost Immunity",
		"of Resist Acid",
		"of Resist Electricity",
		"of Resist Fire",
		"of Resist Cold",
		"of Resist Poison",
		"of Resist Fear",
		"of Resist Light",
		"of Resist Darkness",
		"of Resist Blindness",
		"of Resist Confusion",
		"of Resist Sound",
		"of Resist Shards",
		"of Resist Nexus",
		"of Resist Nether",
		"of Resist Chaos",
		"of Resist Disenchantment"
	},

	/* TR3_ */
	{
		"of Slow Digestion",
		"of Feather Falling",
		"(Glowing)",
		"of Health Regeneration",
		"of Telepathy",
		"of See Invisible",
		"of Free Action",
		"of Hold Life",
		"of Sense Demons",
		"of Sense Dragons",
		"of Sense Giants",
		"of Sense Orcs",
		"of Sense Trolls",
		"of Sense Undead",
		"of Sense Nature",
		"of Mana Regeneration",
		"(Drains Health)",
		"(Drains Mana)",
		"(Drains Experience)",
		"of Aggravation",
		"", /* Uncontrolled */
		"", /* Activates on blow */
		"", /* Activates */
		"(Blessed)",
		"(Quickfingered)",
		"of Hunger",
		"of Charging",
		"of Throwing",
		"of Hurling",
		"",	/* Light curse */
		"",	/* Heavy curse */
		""	/* Permanently cursed */
	},

	/* TR4_ */
	{
		"of Darkness",
		"of Light",
		"of Light Vulnerability",
		"of Water Vulnerability",
		"of Bloodlust",
		"of Manathirst",
		"of Poison Immunity",
		"of Resist Disease",
		"of Resist Water",
		"of Slay Man",
		"of Slay Elf",
		"of Slay Dwarf",
		"of Anchoring",
		"of Silence",
		"of Static",
		"of Ill Winds",
		"(Animal)",
		"(Evil)",
		"(Ancient)",
		"(Demonic)",
		"(Orcish)",
		"(Troll)",
		"(Giant)",
		"(Dragon)",
		"of Man",
		"(Dwarven)",
		"(Elven)",
		"of Poison Vulnerability",
		"of Acid Vulnerability",
		"of Lightning Vulnerability",
		"of Fire Vulnerability",
		"of Frost Vulnerability"
	}
};

const cptr disease_name[33] =
{
	"pink rot",
	"black brain",
	"wit loss",
	"fumblefinger",
	"illweather",
	"warts",
	"food poisoning",
	"thirst",
	"shards",
	"ringing in ears",
	"green rot",
	"magic mushrooms",
	"slow stumble",
	"black eye",
	"a hangover",
	"faint heart",
	"dementia",
	"bad luck",
	"narcolepsy",
	"being stoned",
	"palsy",
	"time loss",
	"static",
	"leeches",
	"mana mites",
	"black breath",
	"the plague",
	"unnatural parasites",
	"Morgoth's curse",
	"minor ailments",
	"hard to treat viruses",
	"incurable illnesses",
	"magical maladies"
};

/*
 * First column is Mana Cost
 * Second column is multiplier * spellpower (breaths handled separately)
 * Third column is a divider to we can have damage like
 * spellpower * 5/2 using only integer math.
 * 4th column is damage variance
 * 5th column is Optimal Ranges for various spells.
 * 6 is optimal for Breath Weapons, Beams, and Arcs.
 * 3 is hard maximum for Lash/Spit.
 * 0 indicates no range limitation for other spells.
 *
 * This range is considered a preference if d_range in spell_desire is > 0.
 * It is a hard limit if d_range = 0.
 *
 * Note that _RF4 tables are non-constant due to updating the blow effects
 * for each monster type.
 */

/*{Mana_cost,dam_mult,dam_div,dam_var,best_range}*/
byte spell_info_RF4[32][5]=
{
	{0,     0,     0,     0,     0},        /* RF4_BLOW_1 */
	{0,     0,     0,     0,     0},        /* RF4_BLOW_2 */
	{0,     0,     0,     0,     0},        /* RF4_BLOW_3 */
	{0,     0,     0,     0,     0},        /* RF4_BLOW_4 */
	{0,     0,     0,     0,     MAX_SIGHT},/* RF4_ADD_AMMO */
	{0,     0,     0,     0,     8},        /* RF4_QUAKE */
	{0,     0,     0,     0,     1},        /* RF4_EXPLODE */
	{0,     0,     0,     0,     1},        /* RF4_AURA */  /* Last spell with fixed maximum range */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_ACID */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_ELEC */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_FIRE */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_COLD */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_POIS */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_PLAS */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_LITE */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_DARK */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_CONFU */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_SOUND */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_SHARD */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_INER */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_GRAV */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_WIND */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_FORCE */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_NEXUS */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_NETHR */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_CHAOS */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_DISEN */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_TIME */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_MANA */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_HOLY */
	{0,     0,     0,     0,     0},        /* RF4_BRTH_FEAR */
	{0,     0,     0,     0,     0}         /* RF4_BRTH_DISEA */
};

 /*{Mana_cost,dam_mult,dam_div,dam_var,best_range}*/
const byte spell_info_RF5[32][5]=
{
	{4,     4,     1,     6,     6},        /* RF5_BALL_ACID */
	{4,     4,     1,     6,     6},        /* RF5_BALL_ELEC */
	{4,     4,     1,     6,     6},        /* RF5_BALL_FIRE */
	{4,     4,     1,     6,     6},        /* RF5_BALL_COLD */
	{4,     3,     1,     6,     6},        /* RF5_BALL_POIS */
	{5,     3,     1,     6,     6},        /* RF5_BALL_LITE */
	{5,     3,     1,     6,     6},        /* RF5_BALL_DARK */
	{6,     3,     1,     6,     6},        /* RF5_BALL_CONFU */
	{4,     2,     1,     6,     6},        /* RF5_BALL_SOUND */
	{4,     3,     1,     6,     6},        /* RF5_BALL_SHARD */
	{4,     2,     1,     6,     6},        /* RF5_BALL_WIND */
	{5,     3,     1,     4,     4},        /* RF5_BALL_STORM */
	{6,     3,     1,     6,     6},        /* RF5_BALL_NETHR */
	{7,     3,     1,     4,     4},        /* RF5_BALL_CHAOS */
	{7,     3,     1,     8,     8},        /* RF5_BALL_MANA */
	{7,     3,     1,     6,     6},        /* RF5_BALL_WATER */
	{4,     4,     1,     6,     6},        /* RF5_BOLT_ACID */
	{4,     4,     1,     6,     6},        /* RF5_BOLT_ELEC */
	{4,     4,     1,     6,     6},        /* RF5_BOLT_FIRE */
	{4,     4,     1,     6,     6},        /* RF5_BOLT_COLD */
	{4,     3,     1,     6,     6},        /* RF5_BOLT_POIS */
	{5,     3,     1,     6,     6},        /* RF5_BOLT_PLAS */
	{5,     3,     1,     6,     6},        /* RF5_BOLT_ICE */
	{5,     3,     1,     6,     6},        /* RF5_BOLT_WATER */
	{5,     3,     1,     6,     6},        /* RF5_BOLT_NETHER */
	{5,     2,     1,     6,     6},        /* RF5_BOLT_MANA */
	{5,     5,     2,     8,     8},        /* RF5_HOLY_ORB */
	{6,     3,     1,     6,     6},        /* RF5_BEAM_ELEC */
	{6,     4,     1,     6,     6},        /* RF5_BEAM_ICE */
	{7,     3,     1,     6,     6},        /* RF5_BEAM_NETHER */
	{7,     0,     0,     0,     6},        /* RF5_ARC_HFIRE */
	{5,     0,     0,     0,     8}         /* RF5_ARC_FORCE */
};

 /*{Mana_cost,dam_mult,dam_div,dam_var,best_range}*/
const byte spell_info_RF6[32][5]=
{
	{6,     0,     0,     0,     0},        /* RF6_HASTE */
	{0,     0,     0,     0,     0},        /* RF6_ADD_MANA */
	{3,     0,     0,     0,     0},        /* RF6_HEAL */
	{3,     0,     0,     0,     0},        /* RF6_CURE */
	{3,     0,     0,     0,     0},        /* RF6_BLINK */
	{8,     0,     0,     0,     0},        /* RF6_TPORT */
	{3,     0,     0,     0,     0},        /* RF6_INVIS */
	{4,     0,     0,     0,     0},        /* RF6_TELE_SELF_TO */
	{4,     0,     0,     0,     0},        /* RF6_TELE_TO */
	{8,     0,     0,     0,     0},        /* RF6_TELE_AWAY */
	{8,     0,     0,     0,     0},        /* RF6_TELE_LEVEL */
	{4,     0,     0,     0,     0},        /* RF6_WRAITHFORM */
	{1,     0,     0,     0,     0},        /* RF6_DARKNESS */
	{2,     0,     0,     0,     0},        /* RF6_TRAPS */
	{6,     0,     0,     0,     0},        /* RF6_FORGET */
	{2,     0,     0,     0,     0},        /* RF6_DRAIN_MANA */
	{4,     3,     2,     6,     6},        /* RF6_CURSE */
	{4,     0,     0,     0,     0},        /* RF6_DISPEL */
	{3,     3,     2,     6,     6},        /* RF6_MIND_BLAST */
	{4,     0,     0,     0,     0},        /* RF6_ILLUSION */
	{4,     5,     2,     6,     6},        /* RF6_WOUND */
	{2,     0,     0,     0,     0},        /* RF6_BLESS */
	{3,     0,     0,     0,     0},        /* RF6_BERSERK */
	{4,     0,     0,     0,     0},        /* RF6_SHIELD */
	{3,     0,     0,     0,     0},        /* RF6_OPPOSE_ELEM */
	{2,     0,     0,     0,     0},        /* RF6_HUNGER */
	{1,     0,     0,     0,     0},        /* RF6_PROBE */
	{1,     0,     0,     0,     0},        /* RF6_SCARE */
	{3,     0,     0,     0,     0},        /* RF6_BLIND */
	{4,     0,     0,     0,     0},        /* RF6_CONF */
	{5,     0,     0,     0,     0},        /* RF6_SLOW */
	{6,     0,     0,     0,     0}         /* RF6_HOLD */
};

 /*{Mana_cost,dam_mult,dam_div,dam_var,best_range}*/
const byte spell_info_RF7[32][5]=
{
	{12,    0,     0,     0,     0},        /* RF7_S_KIN */ /* Summon - 6 */
	{4,     0,     0,     0,     0},        /* RF7_R_KIN */
	{8,     0,     0,     0,     0},        /* RF7_A_DEAD */
	{10,    0,     0,     0,     0},        /* RF7_S_MONSTER */ /* Summon - 1 */
	{15,    0,     0,     0,     0},        /* RF7_S_MONSTERS */ /* Summon - 8 */
	{5,     0,     0,     0,     0},        /* RF7_R_MONSTER */
	{6,     0,     0,     0,     0},        /* RF7_R_MONSTERS */
	{8,     0,     0,     0,     0},        /* RF7_S_PLANT */
	{10,    0,     0,     0,     0},        /* RF7_S_INSECT */ /* Summon - 6 */
	{12,    0,     0,     0,     0},        /* RF7_S_ANIMAL */ /* Summon - 6 */
	{14,    0,     0,     0,     0},        /* RF7_S_HOUND */ /* Summon - 6 */
	{15,    0,     0,     0,     0},        /* RF7_S_SPIDER */ /* Summon - 6 */
	{10,    0,     0,     0,     0},        /* RF7_S_CLASS */
	{10,    0,     0,     0,     0},        /* RF7_S_RACE */
	{15,    0,     0,     0,     0},        /* RF7_S_GROUP */ /* Summon - 6 */
	{5,     0,     0,     0,     0},        /* RF7_S_FRIEND */ /* Summon - 2 */
	{10,    0,     0,     0,     0},        /* RF7_S_FRIENDS */
	{10,    0,     0,     0,     0},        /* RF7_S_ORC */
	{10,    0,     0,     0,     0},        /* RF7_S_TROLL */
	{10,    0,     0,     0,     0},        /* RF7_S_GIANT */
	{14,    0,     0,     0,     0},        /* RF7_S_DRAGON */ /* Summon - 1 */
	{20,    0,     0,     0,     0},        /* RF7_S_HI_DRAGON */ /* Summon - 8 */
	{6,     0,     0,     0,     0},        /* RF7_A_ELEMENT */
	{6,     0,     0,     0,     0},        /* RF7_A_OBJECT */
	{14,    0,     0,     0,     0},        /* RF7_S_DEMON */ /* Summon - 1 / 2-3 */
	{20,    0,     0,     0,     0},        /* RF7_S_HI_DEMON */ /* Summon - 8 */
	{25,    0,     0,     0,     0},        /* RF7_R_UNIQUE */
	{15,    0,     0,     0,     0},        /* RF7_S_UNIQUE */ /* Summon - 8 */
	{20,    0,     0,     0,     0},        /* RF7_S_HI_UNIQUE */ /* Summon - 8 */
	{12,    0,     0,     0,     0},        /* RF7_S_UNDEAD */ /* Summon - 1 */
	{20,    0,     0,     0,     0},        /* RF7_S_HI_UNDEAD */ /* Summon - 8 */
	{20,    0,     0,     0,     0}         /* RF7_S_WRAITH */ /* Summon - 8 */

};

/*
 * d_base:     base desirability for AI.
 * d_summ:     desriability for AI per monster level
 *                  times 0-3 based on number of clear spaces
 * d_hurt:     desirability for AI per monster spell power
 *                  times 0-3 based on damage taken
 * d_mana:     desirability for AI per monster spell power
 *                  times 0-2 based on mana shortage
 * d_esc:      desirability for AI per monster level
 *                  times 0-3 based on fear, and damage taken
 * d_tact:     desirability for AI per monster level, modified
 *                  times 0-3 based on proximity, min_range, and best_range
 * d_res:      category of 'resistability' checked by monster AI
 *                 for purposes of desirability.
 * d_range:    % of spell desirability retained for each step past 'range'
 */

byte spell_desire_RF4[32][8] =
{
/*     d_base	  d_hurt    d_esc	 d_res				    */
/*	     d_summ	d_mana	  d_tact	   d_range		    */
	{ 40,  0,   0,   5,	0,   0,	   0	  ,  100}, /* RF4_BLOW_1    */
	{ 40,  0,   0,   5,	0,   0,	   0	  ,  100}, /* RF4_BLOW_2    */
	{ 40,  0,   0,   5,	0,   0,    0      ,  100}, /* RF4_BLOW_3    */
	{ 40,  0,   0,   5,	0,   0,    0      ,  100}, /* RF4_BLOW_4    */
	{ 40,  0,   0,  25,	0,   0,	   0      ,  100}, /* RF4_ADD_AMMO */
	{ 40,  0,   0,   5,	0,   0,    0      ,  100}, /* RF4_QUAKE	    */
	{ 20,  0,   0,   5,	0,   0, GF_EXPLODE  ,  100}, /* RF4_EXPLODE   */
	{ 40,  0,   0,   0,	0,   0,    0      ,  100}, /* RF4_AURA      */
	{ 75,  0,   0,   5,	0,   0, GF_ACID   ,   90}, /* RF4_BRTH_ACID */
	{ 75,  0,   0,   5,	0,   0, GF_ELEC  ,   90}, /* RF4_BRTH_ELEC */
	{ 75,  0,   0,   5,	0,   0, GF_FIRE  ,   90}, /* RF4_BRTH_FIRE */
	{ 75,  0,   0,   5,	0,   0, GF_COLD  ,   90}, /* RF4_BRTH_COLD */
	{ 65,  0,   0,   5,	0,   0, GF_POIS  ,   90}, /* RF4_BRTH_POIS */
	{ 65,  0,   0,   5,	0,   0, GF_PLASMA  ,   90}, /* RF4_BRTH_PLAS */
	{ 65,  0,   0,   5,	0,   0, GF_LITE  ,   90}, /* RF4_BRTH_LITE */
	{ 65,  0,   0,   5,	0,   0, GF_DARK  ,   90}, /* RF4_BRTH_DARK */
	{ 65,  0,   0,   5,	0,   0, GF_CONFUSION ,   90}, /* RF4_BRTH_CONFU*/
	{ 65,  0,   0,   5,	0,   0, GF_SOUND ,   90}, /* RF4_BRTH_SOUND*/
	{ 65,  0,   0,   5,	0,   0, GF_SHARD ,   90}, /* RF4_BRTH_SHARD*/
	{ 65,  0,   0,   5,	0,   0,	GF_INERTIA,   90}, /* RF4_BRTH_INER */
	{ 65,  0,   0,   5,	0,   0, GF_GRAVITY,   90}, /* RF4_BRTH_GRAV */
	{ 65,  0,   0,   5,	0,   0,	GF_WIND	  ,  90}, /* RF4_BRTH_WIND */
	{ 65,  0,   0,   5,	0,   0, GF_FORCE,   90}, /* RF4_BRTH_FORCE*/
	{ 65,  0,   0,   5,	0,   0, GF_NEXUS ,   90}, /* RF4_BRTH_NEXUS*/
	{ 65,  0,   0,   5,	0,   0, GF_NETHER ,   90}, /* RF4_BRTH_NETHR*/
	{ 65,  0,   0,   5,	0,   0, GF_CHAOS ,   90}, /* RF4_BRTH_CHAOS*/
	{ 65,  0,   0,   5,	0,   0, GF_DISENCHANT ,   90}, /* RF4_BRTH_DISEN*/
	{ 65,  0,   0,   5,	0,   0,	GF_TIME	  ,   90}, /* RF4_BRTH_TIME */
	{ 65,  0,   0,   5,	0,   0,	GF_MANA	  ,   90}, /* RF4_BRTH_MANA */
	{ 65,  0,   0,   5,	0,   0,	GF_HOLY_ORB  ,  90}, /* RF4_BRTH_HOLY */
	{ 65,  0,   0,   5,	0,   0,	GF_TERRIFY  ,  90}, /* RF4_BRTH_FEAR */
	{ 65,  0,   0,   5,	0,   0,	GF_DISEASE	  ,  90}  /* RF4_BRTH_DISEASE */
};

const byte spell_desire_RF5[32][8] =
{
/*     d_base	  d_hurt    d_esc	 d_res				    */
/*	     d_summ	d_mana	  d_tact	   d_range		    */
	{ 50,  0,   0,   0,	0,   0, GF_ACID  ,  100}, /* RF5_BALL_ACID */
	{ 50,  0,   0,   0,	0,   0, GF_ELEC  ,  100}, /* RF5_BALL_ELEC */
	{ 50,  0,   0,   0,	0,   0, GF_FIRE  ,  100}, /* RF5_BALL_FIRE */
	{ 50,  0,   0,   0,	0,   0, GF_COLD  ,  100}, /* RF5_BALL_COLD */
	{ 50,  0,   0,   0,	0,   0, GF_POIS  ,  100}, /* RF5_BALL_POIS */
	{ 40,  0,   0,   0,	0,   0, GF_LITE  ,  100}, /* RF5_BALL_LITE */
	{ 40,  0,   0,   0,	0,   0, GF_DARK  ,  100}, /* RF5_BALL_DARK */
	{ 40,  0,   0,   0,	0,   0, GF_CONFUSION ,  100}, /* RF5_BALL_CONFU*/
	{ 40,  0,   0,   0,	0,   0, GF_SOUND ,  100}, /* RF5_BALL_SOUND*/
	{ 40,  0,   0,   0,	0,   0, GF_SHARD ,  100}, /* RF5_BALL_SHARD*/
	{ 40,  0,   0,   0,	0,   0,	GF_WIND	  ,  100}, /* RF5_BALL_WIND */
	{ 40,  0,   0,   0,	0,   0, GF_STORM ,  100}, /* RF5_BALL_STORM*/
	{ 40,  0,   0,   0,	0,   0, GF_NETHER ,  100}, /* RF5_BALL_NETHR*/
	{ 40,  0,   0,   0,	0,   0, GF_CHAOS ,  100}, /* RF5_BALL_CHAOS*/
	{ 40,  0,   0,   0,	0,   0,	GF_MANA,  100}, /* RF5_BALL_MANA */
	{ 40,  0,   0,   0,	0,   0, GF_WATER ,  100}, /* RF5_BALL_WATER*/
	{ 40,  0,   0,   0,	0,   0, GF_ACID  ,  100}, /* RF5_BOLT_ACID */
	{ 40,  0,   0,   0,	0,   0, GF_ELEC  ,  100}, /* RF5_BOLT_ELEC */
	{ 40,  0,   0,   0,	0,   0, GF_FIRE  ,  100}, /* RF5_BOLT_FIRE */
	{ 40,  0,   0,   0,	0,   0, GF_COLD  ,  100}, /* RF5_BOLT_COLD */
	{ 40,  0,   0,   0,	0,   0, GF_POIS  ,  100}, /* RF5_BOLT_POIS */
	{ 50,  0,   0,   0,	0,   0, GF_PLASMA  ,  100}, /* RF5_BOLT_PLAS */
	{ 50,  0,   0,   0,	0,   0, GF_ICE	  ,  100}, /* RF5_BOLT_ICE  */
	{ 35,  0,   0,   0,	0,   0, GF_WATER ,  100}, /* RF5_BOLT_WATER*/
	{ 35,  0,   0,   0,	0,   0, GF_NETHER ,  100}, /* RF5_BOLT_NETHR*/
	{ 30,  0,   0,   0,	0,   0,	GF_MANA	  ,  100}, /* RF5_BOLT_MANA */
	{ 60,  0,   0,   0,	0,   0, GF_HOLY_ORB  ,  100}, /* RF5_HOLY_ORB*/
	{ 50,  0,   0,   0,	0,   0, GF_ELEC  ,   90}, /* RF5_BEAM_ELEC */
	{ 50,  0,   0,   0,	0,   0, GF_ICE	  ,   90}, /* RF5_BEAM_ICE  */
	{ 50,  0,   0,   0,	0,   0, GF_NETHER ,   90}, /* RF5_BEAM_NETHR*/
	{ 50,  0,   0,   0,	0,   0,	GF_HELLFIRE,  80},  /* RF5_ARC_HFIRE*/
	{ 40,  0,   0,   0,	0,   0,	GF_FORCE  ,  90} 	/* RF5_ARC_FORCE */
};


const byte spell_desire_RF6[32][8] =
{
/*     d_base	  d_hurt    d_esc	 d_res				    */
/*	     d_summ	d_mana	  d_tact	   d_range		    */
	{ 50,  0,   0,   0,	5,   5,	   0	  ,  100}, /* RF6_HASTE	    */
	{ 15,  0,   0,  25,	0,   0,	   0  ,  100}, /* RF6_ADD_MANA  */
	{ 10,  0,   30,  0,	0,   0,	   0	  ,  100}, /* RF6_HEAL	    */
	{ 50,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_CURE	    */
	{ 27,  0,   0,   0,	10,  15,   0	  ,  100}, /* RF6_BLINK	    */
	{  3,  0,   0,   0,	20,  10,   0	  ,  100}, /* RF6_TPORT	    */
	{ 50,   0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_INVIS	    */
	{ 30,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_TELE_SELF_TO*/
	{ 30,  0,   0,   0,	0,   10,   0	  ,  100}, /* RF6_TELE_TO   */
	{  3,  0,   0,   0,	20,  10,   0	  ,  100}, /* RF6_TELE_AWAY */
	{  3,  0,   0,   0,	20,  10,   0 ,	   100}, /* RF6_TELE_LEVEL */
	{ 50,   0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_WRAITHFORM*/
	{ 20,  0,   0,   0,	5,   0,	   0	  ,  100}, /* RF6_DARKNESS  */
	{ 25,  0,   0,   0,	5,   0,	   0	  ,  100}, /* RF6_TRAPS	    */
	{ 45,  0,   0,   0,	0,   0, 0  ,  100}, /* RF6_FORGET    */
	{ 25,  0,   0,   15,	0,   0, GF_LOSE_MANA,  100}, /* RF6_DRAIN_MANA*/
	{ 45,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_CURSE	    */
	{ 45,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_DISPEL   */
	{ 30,  0,   0,   0,	0,   0, 0  ,  100}, /* RF6_MIND_BLAST*/
	{ 30,  0,   0,   0,	0,   0, GF_HALLU,  100}, /* RF6_ILLUSION*/
	{ 40,  0,   0,   0,	0,   0, GF_HURT,  100}, /* RF6_WOUND	    */
	{ 50,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_BLESS	    */
	{ 35,  0,   0,   10,	0,   0,	   0	  ,  100}, /* RF6_BERSERK    */
	{ 50,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_SHIELD    */
	{ 50,  0,   0,   0,	0,   0,	   0	  ,  100}, /* RF6_OPPOSE_ELEM */
	{ 25,  0,   0,   0,	0,   0,	GF_HUNGER,  100}, /* RF6_HUNGER    */
	{ 50,  0,   0,   0,	0,   0,	GF_PROBE	  ,  100}, /* RF6_PROBE	    */
	{ 25,  0,   0,   0,	0,   0, GF_FEAR_WEAK,	  100}, /* RF6_SCARE	 */
	{ 30,  0,   0,   0,	0,   0, GF_BLIND_WEAK,	   100}, /* RF6_BLIND	  */
	{ 30,  0,   0,   0,	0,   0, GF_CONF_WEAK,	   100}, /* RF6_CONF	  */
	{ 40,  0,   0,   0,	0,   0, GF_SLOW_WEAK,	  100}, /* RF6_SLOW	 */
	{ 35,  0,   0,   0,	0,   0, GF_SLEEP,	  100} /* RF6_HOLD	*/
};

const byte spell_desire_RF7[32][8] =
{
 /*     d_base	  d_hurt    d_esc	 d_res				    */
 /*	     d_summ	d_mana	  d_tact	   d_range		    */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_KIN	    */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_R_KIN	    */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_A_DEAD    */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_MONSTER */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_MONSTERS*/
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_R_MONSTER */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_R_MONSTERS*/
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_PLANT   */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_INSECT  */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_ANIMAL  */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_HOUND   */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_SPIDER  */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_CLASS   */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_RACE    */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_GROUP   */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_FRIEND  */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_FRIENDS */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_ORC     */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_TROLL   */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_GIANT   */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_DRAGON  */
	{ 0,   17,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_HI_DRAGON*/
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_A_ELEMENT */
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_A_OBJECT  */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_DEMON   */
	{ 0,   17,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_HI_DEMON*/
	{ 0,   15,   0,   0,	0,   0,	   0	  ,  100}, /* RF7_R_UNIQUE  */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_UNIQUE  */
	{ 0,   18,  0,   0,	0,   0,	   0  	  ,  100}, /* RF7_S_HI_UNIQUE */
	{ 0,   15,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_UNDEAD  */
	{ 0,   17,  0,   0,	0,   0,	   0	  ,  100}, /* RF7_S_HI_UNDEAD*/
	{ 0,   18,  0,   0,	0,   0,	   0	  ,  100}  /* RF7_S_WRAITH  */
};


/*
 * Define which element the monster belongs to.
 *
 * Elements are Fire, Water, Earth, Air, Magma, Ice, Ooze, Smoke.
 *
 */
const element_type element[MAX_ELEMENTS] =
{
	/* ELEM_FIRE */
	{GF_FIRE, 0L, 14, 0},

	/* ELEM_EARTH */
	{0, (FF2_CAN_DIG), 13, 82},

	/* ELEM_AIR */
	{GF_WIND, (FF2_CHASM), 11, 0},

	/* ELEM_WATER */
	{GF_WATER, (FF2_WATER | FF2_CAN_SWIM), 12, 224},

	/* ELEM_MAGMA */
	{GF_LAVA, (FF2_LAVA), 19, 0},

	/* ELEM_OOZE */
	{GF_ACID, (FF2_ACID), 16, 0},

	/* ELEM_ICE */
	{GF_ICE, (FF2_ICE), 18, 0},

	/* ELEM_SMOKE */
	{GF_SMOKE, (FF2_OIL), 17, 0}
};

/*
 * Define which items all starting characters get.
 */
const start_item common_items[MAX_COMMON_ITEMS]=
{
	{	TV_FOOD, SV_FOOD_PINT_OF_SPIRITS, 1, 1, 0, 0, 0, 15},
	{	TV_FOOD, SV_FOOD_PINT_OF_ALE, 1, 1, 0, 0, 16, 60},
	{	TV_FOOD, SV_FOOD_PINT_OF_WINE, 1, 1, 0, 0, 61, 95},
	{	TV_FOOD, SV_FOOD_APPLE, 1, 1, 0, 0, 96, 100},
	{	TV_FOOD, SV_FOOD_SLIME_MOLD, 6, 7, 0, 0, 0, 20},
	{	TV_FOOD, SV_FOOD_BISCUIT, 13, 19, 0, 0, 21, 40},
	{	TV_FOOD, SV_FOOD_RATION, 3, 7, 0, 0, 41, 100},
	{	TV_LITE, SV_LITE_TORCH, 3, 7, 2500, 3500, 0, 75},
	{	TV_LITE, SV_LITE_LANTERN, 1, 1, 500, 3500, 76, 100},
	{	TV_FLASK, SV_FLASK_OIL, 2, 5, 0, 0, 76, 100},
	{	TV_HELM, SV_HARD_LEATHER_CAP, 1, 1, 0, 0, 60, 99},
	{	TV_CROWN, SV_GOLDEN_CROWN, 1, 1, 0, 0, 100, 100},
	{	TV_BOOTS, SV_PAIR_OF_HARD_LEATHER_BOOTS, 1, 1, 0, 0, 50, 70},
	{	TV_BOOTS, SV_PAIR_OF_SOFT_LEATHER_BOOTS, 1, 1, 0, 0, 71, 100},
	{	TV_GLOVES, SV_SET_OF_LEATHER_GLOVES, 1, 1, 0, 0, 91, 100},
	{	TV_CLOAK, SV_CLOAK, 1, 1, 0, 0, 81, 100}
};

const cptr vocalize[MAX_LANGUAGES] =
{
	"says",		/* LANG_COMMON */
	"says",		/* LANG_ELF */
	"says",		/* LANG_DWARF */
	"barks",	/* LANG_ORC */
	"roars",	/* LANG_TROLL */
	"thunders",	/* LANG_GIANT */
	"roars",	/* LANG_DRAGON */
	"utters",	/* LANG_DEMON */
	"whispers",	/* LANG_UNDEAD */
	"thinks",	/* LANG_ESP */
	"creaks",	/* LANG_FOREST */
	"releases",	/* LANG_MUSHROOM */
	"says",		/* LANG_NATURAL */
	"roars", 	/* 'A' */
	"squawks",
	"growls",
	"roars",
	"shudders",
	"croaks", 	/* 'F' */
	"says",
	"roars",
	"chirps",
	"hisses",
	"clicks", 	/* 'K' */
	"whispers",
	"says",
	"whispers",
	"barks",
	"thunders", /* 'P' */
	"grunts",
	"hisses",
	"says",
	"roars",
	"utters", 	/* 'U' */
	"whispers",
	"whispers",
	"clacks",
	"hoots",
	"howls", 	/* 'Z' */
	"releases", 	/* 'a' */
	"screeches",
	"clicks",
	"roars",
	"blinks",
	"growls", 	/* 'f' */
	"beeps",
	"says",
	"squelches",
	"squelches",
	"barks", 	/* 'k' */
	"says",
	"releases",
	"hisses",
	"barks",
	"says", 	/* 'p' */
	"says",
	"screeches",
	"whispers",
	"says",
	"utters", 	/* 'u' */
	"drones",
	"writhes",
	"says",
	"hisses",
	"drools"	/* 'z' */
};



/*
 * Months from T.o.M.E
 */
const int month_day[9] =
{
	0,        /* 1 day */

	1,        /* 54 days */
	55,       /* 72 days */
	127,      /* 54 days */

	181,      /* 3 days */

	184,      /* 54 days */
	238,      /* 72 days */
	310,      /* 54 days */

	364,      /* 1 day */
};


/*
 * Month names from T.o.M.E
 */
const cptr month_name[9] =
{
	"Yestare",

	"Tuile",
	"Laire",
	"Yavie",

	"Enderi",

	"Quelle",
	"Hrive",
	"Coire",

	"Mettare",
};


/*
 * Bag definitions
 *
 * A bag is a collection of items uniquely identified by their tval and sval.
 *
 * A bag can hold either one or (hack) two further distinguishing attributes:
 * the number of such items in the bag, and (possibly) the total number of
 * charges. The total number of charges is a hack -- because we either use
 * a second 'fake' bag entry to track the charges (for wands), or abstract
 * the total number of items out and just track charges (for torches).
 *
 * This means we can have bags for potions, food, spikes, etc. but not rods
 * (because they can have a timeout), bodies (because they are also distinguished
 * by the monster type), weapons and armour (because they have many other
 * variables tracked), lanterns (because we don't want to apply the hack to
 * lanterns).
 *
 * XXX XXX We accomadate a small information loss: if a player notices
 * something about a kind of item, such as the fact a wand of fire ignores
 * fire, this information will be lost when the item put in a bag. The
 * alternative is to only let the player put identified items in the bag, but
 * this makes bags a lot less useful.
 *
 * XXX XXX To provide some kind of balance, we do not create bags for spellbooks.
 *
 * XXX XXX We restrict bags to holding 22 different kinds of items because this
 * is the maximum number we can display in a list on an 80x24 screen.
 *
 * We treat these bags as 'magic' and track the contents once - every bag of
 * the same type has 'magical' access to the same contents as every other
 * bag.
 */
const s16b bag_holds[SV_BAG_MAX_BAGS][INVEN_BAG_TOTAL][2] =
{
	/* Bag of Poisons - holds dangerous potions */
	{ {75, 4}, {75, 5}, {75, 7}, {75, 9},
	  {75, 11}, {75, 13}, {75, 15}, {75, 16},
	  {75, 17}, {75, 18}, {75, 19}, {75, 20},
	  {75, 21}, {75, 22}, {75, 73},	{75, 74},
	  {75, 76}, {75, 77}, {75, 78}, {75, 79},
	  {75, 80}, {75, 81}, {75, 85} },
	/* Bag of Harmful Mushrooms - holds harmful mushrooms */
	{ {79, 0}, {79, 1}, {79, 2}, {79, 3},
	  {79, 4}, {79, 5}, {79, 10}, {79, 11},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Black Magics -- holds cursed scrolls */
	{ {70, 0}, {70, 1}, {70, 2}, {70, 3},
	  {70, 4}, {70, 7}, {70, 5}, {70, 69},
	  {70, 74}, {70, 75}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of supplies -- torches, oil, spikes etc. Torches are handed specially. */
	{ {77, 0}, {2, 2}, {77, 2}, {39, 0},
	  {5, 0}, {2, 1}, {2, 3}, {2, 4},
	  {14, 1}, {14, 2}, {14, 4}, {77, 7},
	  {77, 8}, {77, 9}, {77, 10}, {77, 11},
	  {77, 12}, {77, 13}, {77, 14}, {77, 15},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of provisions -- foods of various types */
	{ {80, 30}, {80, 32}, {80, 33}, {80, 35},
	  {80, 36},	{80, 37}, {80, 38}, {80, 39},
	  {80, 40}, {80, 41}, {80, 42}, {75, 1},
	  {75, 2}, {75, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}},
	/* Bag of Antidotes - holds low power potions */
	{ {75, 26}, {75, 27}, {75, 31}, {75, 30},
	  {75, 34}, {75, 32}, {75, 23}, {75, 71},
	  {75, 83}, {75, 54}, {75, 75}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Philtres - holds medium power potions */
	{ {75, 24}, {75, 25}, {75, 36}, {75, 49},
	  {75, 51}, {75, 40}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}  },
	/* Bag of Elixirs - holds high power potions */
	{ {75, 55}, {75, 57}, {75, 58}, {75, 59},
	  {75, 63}, {75, 65}, {75, 66}, {75, 72},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Helpful Mushrooms - holds helpful mushrooms */
	{ {79, 12}, {79, 13}, {79, 14}, {79, 15},
	  {79, 16}, {79, 17}, {79, 18}, {79, 19},
	  {79, 20}, {79, 21}, {79, 22}, {79, 23},
	  {79, 24}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Scrying - holds lesser identify/detection scrolls */
	{ {70, 26}, {70, 27}, {70, 28}, {70, 29},
	  {70, 30}, {70, 19}, {70, 23}, {70, 43},
	  {70, 48}, {70, 65}, {70, 66},	{70, 73},
	  {70, 77}, {70, 78}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Hedge Magics - holds less useful scrolls */
	{ {70, 8}, {70, 9}, {70, 10}, {70, 11},
	  {70, 22}, {70, 24}, {70, 32}, {70, 33},
	  {70, 34}, {70, 35}, {70, 37}, {70, 39},
	  {70, 59}, {70, 60}, {70, 62}, {70, 64},
	  {70, 76}, {70, 79}, {70, 80}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Enchantments - holds enchantment scrolls */
	{ {70, 14}, {70, 16}, {70, 17}, {70, 18},
	  {70, 20}, {70, 21}, {70, 49}, {70, 50},
	  {70, 51}, {70, 53}, {70, 54}, {70, 55},
	  {70, 56}, {70, 57}, {70, 58}, {70, 67},
	  {70, 68}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Eldritch Magics - holds powerful scrolls */
	{ {70, 15}, {70, 38}, {70, 41}, {70, 42},
	  {70, 44}, {70, 45}, {70, 46}, {70, 47},
	  {70, 63}, {70, 70}, {70, 71}, {70, 72},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Lesser Runes - holds weaker runes */
	{ {93, 1}, {93, 3}, {93, 5}, {93, 7},
	  {93, 11}, {93, 12}, {93, 13}, {93, 14},
	  {93, 15}, {93, 16}, {93, 19}, {93, 21},
	  {93, 22}, {93, 25}, {93, 26}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Greater Runes - holds more powerful runes */
	{ {93, 2}, {93, 4}, {93, 6}, {93, 8},
	  {93, 9}, {93, 10}, {93, 17}, {93, 18},
	  {93, 20}, {93, 23}, {93, 24}, {93, 27},
	  {93, 28}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Maps - holds maps */
	{ {72, 25}, {72, 31}, {72, 41}, {72, 47},
	  {72, 50}, {72, 56},  {72, 62}, {72, 64},
	  {72, 65}, {72, 66}, {72, 68}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of bewitchments -- low power non-elemental wands */
	{ {65, 0}, {65, 1}, {65, 2}, {65, 4},
	  {65, 7}, {65, 8}, {65, 9}, {65, 10},
	  {65, 11}, {65, 12}, {65, 13}, {65, 24},
	  {65, 32}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* XXX XXX Hack -- Wand bags need two slots */
	{ {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of elements -- low power elemental wands */
	{ {65, 5}, {65, 6}, {65, 14}, {65, 15},
	  {65, 16}, {65, 17}, {65, 18}, {65, 19},
	  {65, 20}, {65, 21}, {65, 22}, {65, 23},
	  {65, 33}, {65, 34}, {65, 36}, {65, 38},
	  {65, 39}, {65, 40}, {65, 42}, {65, 43},
	  {65, 67}, {0, 0}, {0, 0} },
	/* XXX XXX Hack -- Wand bags need two slots */
	{ {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of four winds -- medium power wands */
	{ {65, 3}, {65, 26}, {65, 27}, {65, 28},
	  {65, 29}, {65, 30}, {65, 34}, {65, 35},
	  {65, 37}, {65, 41}, {65, 45}, {65, 46},
	  {65, 47}, {65, 48}, {65, 49}, {65, 50},
	  {65, 53}, {65, 54}, {65, 55}, {65, 69},
	  {65, 70}, {65, 71}, {0, 0} },
	/* XXX XXX Hack -- Wand bags need two slots */
	{ {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of six demons -- high power wands */
	{ {65, 25}, {65, 27}, {65, 28}, {65, 31},
	  {65, 44}, {65, 51}, {65, 52}, {65, 56},
	  {65, 57}, {65, 58}, {65, 59}, {65, 60},
	  {65, 61}, {65, 62}, {65, 63}, {65, 64},
	  {65, 65}, {65, 66}, {65, 67}, {65, 68},
	  {0, 0}, {0, 0}, {0, 0} },
	/* XXX XXX Hack -- Wand bags need two slots */
	{ {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },

	/* More bags */

	/* Bag of Mixed Mushrooms - holds mushrooms with mixed abilities */
	{ {79, 6}, {79, 7}, {79, 8}, {79, 9},
	  {79, 27}, {79, 28}, {79, 29}, {79, 30},
	  {79, 25}, {79, 26}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Reagents - holds low-medium power potions */
	{ {75, 35}, {75, 41}, {75, 47}, {75, 60},
	  {75, 61}, {75, 62}, {75, 64}, {75, 68},
	  {75, 69}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Serums - holds medium power potions */
	{ {75, 33}, {75, 48}, {75, 52}, {75, 29},
	  {75, 67}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}  },
	/* Bag of Panaceas - holds high power potions */
	{ {75, 37}, {75, 38}, {75, 39}, {75, 56},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} },
	/* Bag of Divination - holds greater identify/detection scrolls */
	{ {70, 25}, {70, 31}, {70, 40}, {70, 61},
	  {70, 12}, {70, 13}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0}, {0, 0},
	  {0, 0}, {0, 0}, {0, 0} }
};



/*
 * Names of various styles
 */
const cptr w_name_style[32] =
{
	"None",
	"Unarmed",
	"One-handed",
	"Two-handed",
	"Shield & weapon",
	"Dual-wield",
	"Hafted weapons",
	"Swords",
	"Polearms",
	"Thrown weapons",
	"Slings",
	"Bows",
	"Cross-bows",
	"Backstab",
	"Magic books",
	"Prayer books",
	"Song books",
	"Instruments",
	"Potions",
	"Scrolls",
	"Amulets",
	"Rings",
	"Wands/Rods",
	"Staves",
	"Slay Orcs",
	"Slay Trolls",
	"Slay Giants",
	"Slay Dragons",
	"Slay Evil",
	"Slay Undead",
	"Slay Animals",
	"Slay Demons"
};


/*
 * Tval associated with each style
 */
const s16b style2tval[32] =
{
	0,
	0,
	0,
	0,
	TV_SHIELD,
	0,
	TV_HAFTED,
	TV_SWORD,
	TV_POLEARM,
	0,
	TV_SHOT,
	TV_ARROW,
	TV_BOLT,
	0,
	TV_MAGIC_BOOK,
	TV_PRAYER_BOOK,
	TV_SONG_BOOK,
	TV_INSTRUMENT,
	TV_POTION,
	TV_SCROLL,
	TV_AMULET,
	TV_RING,
	TV_WAND,
	TV_STAFF,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0
};

/*
 * Race of parasites
 *
 * This is a temporary hack.
 */
const s16b parasite_hack[DISEASE_BLOWS] =
{
	646,	/* Pink mushroom patch */
	885,	/* Brain ooze */
	107,	/* Lost soul */
	65,	/* Yellow worm mass */
	664,	/* Purple slime */
	92,	/* Brown mold */
	897,	/* Brown icky thing -- nothing like scatalogical in-jokes */
	895,	/* Yellow icky thing */
	219,	/* Earth spirit */
	234,	/* Giant gold dragonfly */
	648,	/* Green mushroom patch */
	191,	/* Magic mushroom patch */
	141,	/* Giant white tick */
	588,	/* Black eye */
	32,	/* Shrieker mushroom patch */
	835,	/* Shiver */
	119,	/* Lemure */
	858,	/* Crying spirit */
	1247,	/* Sheep -- narcolepsy */
	219,	/* Earth spirit */
	659,	/* White slime */
	656,	/* Wrinkled mushroom */
	85	/* Disenchanter eye */
};


const cptr cause_of_death[-SOURCE_PLAYER_END][SOURCE_MESSAGES] =
{
	{ "","","",""},	/* SOURCE_SELF */
	{" being destroyed under you"," being destroyed under you"," being destroyed under you"," being destroyed under you"},	/* SOURCE_OBJECT */
	{"", "", "", ""}, /* SOURCE_FEATURE */
	{"", "", "", ""}, /* SOURCE_SPELL */
	{"", "", "", ""}, /* SOURCE_DISEASE */
	{"daylight", "the sun rising", "no UV protection", "your unholy flesh searing in the radiance of day"}, /* SOURCE_DAYLIGHT */
	{" being born", " hatching", " emerging", " erupting from you"},	/* SOURCE_BIRTH */
	{"an untreated scratch", "a nasty cut", "a fatal wound", "poor bandaging"},	/* SOURCE_CUTS */
	{"poison", "venom", "toxin", "unnatural swelling"},	/* SOURCE_POISON */
	{"hunger", "starvation", "lack of food", "poor management"},	/* SOURCE_HUNGER */
	{"an ancient curse", "a perpetual weakness", "a black pact", "the spoor of Morgoth"},	/* SOURCE_CURSED_ITEM */
	{"being entombed by ", "being crushed by ", "being buried under ", "a falling cask of Amontillado stored by "},	/* SOURCE_ENTOMB */
	{"blood debt for a slain ", "backlash for summoning a ", "a dark pact claimed by a dying ", "unearthly powers seeking vengeance on behalf of a "}, /* SOURCE_BLOOD_DEBT */
	{"misjudging ", "treachery of ", "a knife in the back from ", "walking in front of the line of fire of "}, /* SOURCE_PLAYER_ALLY */
	{"a swing, and a miss", "overenthusiasm", "a hellavu shaving cut", "hmmm... you're not quite sure how you hit that part of you"}, /* SOURCE_PLAYER_ATTACK */
	{"a misfire","an unintended discharge","shooting your own foot","hmmm... you're not quite sure how you shot that part of you"}, /* SOURCE_PLAYER_SHOT */
	{"a misthrow", "a fumble","juggling a dangerous object","hmmm... you're not quite sure how you threw that there"}, /* SOURCE_PLAYER_THROW */
	{"walking headfirst into a ","walking backwards into a ","stumbling into a ","forgetting where you put a "}, /* SOURCE_PLAYER_TRAP */
	{"breaking ", "smashing ", "cracking open ", "shattering "}, /* SOURCE_PLAYER_BREAK */
	{"sniffing ", "inhaling ", "snorting ", "getting addicted to "}, /* SOURCE_PLAYER_SPORE */
	{"a nick from a weapon coated with ", "extract of ", "toxic ", "fumes reeking of "		}, /* SOURCE_PLAYER_COATING */
	{"over cooking ", "undercooking ", "having a taste for ", "not cutting out the poisonous bits off "}, /* SOURCE_PLAYER_EAT_MONSTER */
	{"trying to draw health from ", "the eldritch power of ", "taking the place of ", "connecting to the Negative Plane through "}, /* SOURCE_PLAYER_VAMP_DRAIN */
	{"nibbling ", "tasting ", "trying ", "an unknown allergy to "}, /* SOURCE_PLAYER_EAT_UNKNOWN */
	{"sipping ", "sniffing ", "fumes from ", "not spitting out "}, /* SOURCE_PLAYER_QUAFF_UNKNOWN */
	{"glancing at ", "casting your eyes over ", "browsing ", "looking sideways at "}, /* SOURCE_PLAYER_READ_UNKNOWN */
	{"eating ", "chewing ", "choking on ", "indigestion from "}, /* SOURCE_PLAYER_EAT */
	{"quaffing ", "drinking ", "slurping ", "guzzling "}, /* SOURCE_PLAYER_QUAFF */
	{"aiming ", "waving ", "pointing ", "holding the wrong end of "}, /* SOURCE_PLAYER_AIM */
	{"zapping ", "plughing ", "xyzzying ", "holding the wrong end of "}, /* SOURCE_PLAYER_ZAP */
	{"rubbing ", "wondering how to zap ", "seeing which end glows of ", "pressing a hidden stud on "}, /* SOURCE_PLAYER_ZAP_NO_TARGET */
	{"reading ", "misreading ", "reciting ", "holding upside down "}, /* SOURCE_PLAYER_READ */
	{"using ", "misusing ", "abusing ", "badly using "}, /* SOURCE_PLAYER_USE */
	{"activating ", "relying too heavily on ", "mistrusting the power of ", "chancing the gods with "}, /* SOURCE_PLAYER_ACT_ARTIFACT */
	{"activating ", "relying on ", "having an argument with ", "chancing fate with "}, /* SOURCE_PLAYER_ACT_EGO_ITEM */
	{"activating ", "calling on ", "discharging ", "misusing "}, /* SOURCE_PLAYER_ACTIVATE */
	{"buying ", "acquiring ", "purchasing ", "the ill-judged purchase of "}, /* SOURCE_PLAYER_SERVICE */
	{"if you died here, you shouldn't be using wizard commands", "are you reading the source again?", "hmmph", "i'm going to ROT13 this table"}, /* SOURCE_PLAYER_WIZARD */
	{"bad luck casting ", "badly casting ", "slurring a syllable casting ", "magical feedback whilst casting "} /* SOURCE_PLAYER_CAST */
};


/*
 * Used for timed condition checking below.
 */
#define TMD_CONDITION(T) \
	((T) < TMD_CONDITION_MAX ? 1L << (T) : 0L)

/*
 * Item manipulation commands.
 *
 * TODO: We could combine player_eat_food & player_quaff potion to a 'player_consume'
 * function and player_use_staff and player_aim_wand to a 'player_use_charge' function etc.
 */
const do_cmd_item_type cmd_item_list[MAX_COMMANDS] =
{
	{ NULL, '\0', "nothing", " which ? ", "You have nothing to .",
			NULL, 0, (USE_INVEN | USE_FLOOR | USE_FEATU), 0L, 0L, 0, NULL},

	{ player_eat_food, 'E', "Eat", "Eat which item? ", "You have nothing to eat.",
			item_tester_hook_food_edible, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_BAGC | USE_ALLY | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_quaff_potion, 'q', "Quaff", "Quaff which potion? ", "You have nothing to quaff.",
			NULL, TV_POTION, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_BAGC | USE_ALLY | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_read_scroll, 'r', "Read", "Read which scroll? ", "You have no scrolls to read.",
			NULL, TV_SCROLL, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_BAGC | USE_ALLY | USE_HANDLE), (CONDITION_LITE | CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK) | TMD_CONDITION(TMD_BLIND), 0, NULL},

	{ player_use_staff, 'u', "Use", "Use which staff? ", "You have no staffs to use.",
			NULL, TV_STAFF, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATU | USE_ALLY | USE_HANDLE), (CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_aim_wand, 'u', "Aim", "Aim which wand? ", "You have no wands to aim.",
			NULL, TV_WAND, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_BAGC | USE_ALLY | USE_HANDLE), (CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_zap_rod, 'z', "Zap", "Zap which rod? ", "You have no rods to zap.",
			item_tester_hook_rod_charged, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_ALLY | USE_HANDLE), (CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_assemble, 'Y', "Assemble", "Assemble which item? ", "You have no items to assemble.",
			item_tester_hook_assemble, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_ALLY | USE_HANDLE), 0L, 0L, COMMAND_ITEM_ASSEMBLY, NULL},

	{ player_assembly, 'Y', "Assemble with", "Assemble with which item? ", "You have no items to assemble it with.",
			item_tester_hook_assembly, 0, (USE_INVEN | USE_FLOOR | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_activate, 'A', "Activate", "Activate which item? ", "You have no items you know how to activate.",
			item_tester_hook_activate, 0, (USE_EQUIP | USE_SELF), (CONDITION_NO_SNEAKING), 0L, 0, NULL},

	{ player_apply_rune_or_coating, 'y', "Apply", "Apply which runestone, mushroom, flask or potion? ", "You have no runestones, mushrooms, flasks or potions.",
			item_tester_hook_apply, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_BAGC | USE_ALLY | USE_HANDLE), 0L, 0L, 0, cmd_tester_rune_or_coating},

	{ player_apply_rune_or_coating2, 'Y', "Apply to", "Apply rune to which item? ", "You have nothing to apply runes to.",
			NULL, 0, (USE_INVEN | USE_EQUIP | USE_FLOOR | USE_SKIN | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_apply_rune_or_coating2, 'y', "Coat", "Coat which item? ", "You have nothing which will hold a coating.",
			item_tester_hook_coating, 0, (USE_INVEN  | USE_EQUIP | USE_FLOOR | USE_SKIN | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_wield, 'w', "Wear or wield", "Wear/Wield which item? ", "You have nothing you can wear or wield.",
			item_tester_hook_wear, 0, (USE_INVEN | USE_FLOOR | USE_FEATG | USE_BAGC | USE_HANDLE), 0L, 0L, 0, NULL},
#if 0
	{ player_wield, 'W', "Wield", "Wield which item? ", "You have nothing you can wield.",
			item_tester_hook_wear /*_wield*/, 0, (USE_INVEN | USE_FLOOR | USE_FEATG | USE_BAGC | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_wield, 'w', "Wear", "Wear which item? ", "You have nothing you can wear.",
			item_tester_hook_wear /*_wear*/, 0, (USE_INVEN | USE_FLOOR | USE_FEATG | USE_BAGC | USE_HANDLE), 0L, 0L, 0, NULL},
#endif
	{ player_wield /*_offhand*/, 'o', "Wield into off-hand", "Wield which item into your off-hand? ", "You have nothing you can wear or wield in your off-hand.",
			item_tester_hook_wear /*_offhand */, 0, (USE_EQUIP | USE_FLOOR | USE_FEATG), 0L, 0L, 0, NULL},

	{ player_wield /*_quiver*/, 'p', "Put into quiver", "Put which item into your quiver? ", "You have nothing you can put in your quiver.",
			item_tester_hook_wear /*_quiver*/, 0, (USE_EQUIP | USE_FLOOR | USE_FEATG | USE_BAGC), 0L, 0L, 0, NULL},

	{ player_takeoff, 't', "Take off", "Take off which item? ", "You are not wearing anything to take off.",
			item_tester_hook_removable, 0, (USE_EQUIP | USE_ALLY | USE_RANGE | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_drop, 'd', "Drop", "Drop which item? ", "You have nothing to drop.",
			item_tester_hook_droppable, 0, (USE_EQUIP | USE_INVEN | USE_BAGC | USE_BAGS | USE_ALLY | USE_RANGE /*| USE_GOLD */ | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_destroy, 'k', "Destroy", "Destroy which item? ", "You have nothing to destroy.",
			NULL, 0, (USE_INVEN | USE_FLOOR | USE_FEATG | USE_BAGC | USE_BAGS | USE_ALLY | USE_RANGE | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_observe, 'x', "Examine", "Examine which item? ", "You have nothing to examine.",
			NULL, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATG | USE_BAGC | USE_BAGS | USE_ALLY | USE_TARGET | USE_RANGE | USE_KNOWN | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_uninscribe, '}', "Un-inscribe", "Un-inscribe which item? ", "You have nothing to un-inscribe.",
			NULL, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_ALLY | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_inscribe, '{', "Inscribe", "Inscribe which item? ", "You have nothing to inscribe.",
			NULL, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_ALLY | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_refill, 'F', "Fill or fuel", "Fill/Fuel which item? ", "You have nothing to fill or fuel.",
			item_tester_empty_flask_or_lite, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY | USE_HANDLE), 0L, 0L, 0, cmd_tester_fill_or_fuel},
#if 0
	{ player_refill, 'F', "Fill", "Fill/Fuel which item? ", "You have nothing to fill or fuel.",
			item_tester_empty_flask_or_lite, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY), 0L, 0L, 0, cmd_tester_fill_or_fuel},

	{ player_refill, 'F', "Fuel", "Fill/Fuel which item? ", "You have nothing to fill or fuel.",
			item_tester_empty_flask_or_lite, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY), 0L, 0L, 0, cmd_tester_fill_or_fuel},
#endif
	{ player_refill2, '!', "Fill from", "Fill it from where? ", "You have nothing to fill it with.",
			item_tester_refill_flask, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_FEATU | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_refill2, '~', "Refuel", "Refuel with which torch? ", "You have no torches.",
			item_tester_refill_torch, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_refill2, '`', "Refill lantern", "Refill with which source of oil? ", "You have no sources of oil.",
			item_tester_refill_lantern, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_refill2, '`', "Refill firearm", "Refill with which gunpowder? ", "You have no gunpowder.",
			item_tester_refill_firearm, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_light_and_douse, '|', "Light or douse", "Light/Douse which light source? ", "You have nothing to light or douse.",
			item_tester_light_source, 0, (USE_EQUIP | USE_FLOOR | USE_FEATH | USE_HANDLE), 0L, 0L, 0, NULL},
#if 0
	{ player_light_and_douse, 'l', "Light", "Light which light source? ", "You have nothing to light or douse.",
			item_tester_light_source /*light*/, 0, (USE_EQUIP | USE_FLOOR | USE_FEATH | USE_HANDLE), 0L, 0L, 0, NULL},

	{ player_light_and_douse, 'D', "Douse", "Douse which light source? ", "You have nothing to light or douse.",
			item_tester_light_source /*douse*/, 0, (USE_EQUIP | USE_FLOOR | USE_FEATH | USE_HANDLE), 0L, 0L, 0, NULL},
#endif
	{ player_set_magic_trap, 'S', "Set trap", "Set trap with which item? ", "You have nothing which can be used in a trap.",
			item_tester_hook_magic_trap, 0, (USE_INVEN | USE_FLOOR | USE_ALLY), 0L, 0L, 0, NULL},

	{ player_throw, 't', "Throw", "Throw which item? ", "You have nothing to throw.",
			item_tester_hook_throwable, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATG | USE_BAGC | USE_HANDLE),
			(CONDITION_NO_WIND | CONDITION_SKILL_THROW | CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_fire, 'f', "Sling", "Sling which item? ", "You have nothing to sling around.",
			item_tester_hook_slingable, 0, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATG | USE_HANDLE),
			(CONDITION_NO_WIND | CONDITION_SKILL_FIRE | CONDITION_NO_SNEAKING | CONDITION_SHOOTING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_fire, 'f', "Fire", "Fire which arrow? ", "You have no arrows to fire.",
			0, TV_ARROW, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATG | USE_HANDLE),
			(CONDITION_NO_WIND | CONDITION_SKILL_FIRE | CONDITION_NO_SNEAKING | CONDITION_SHOOTING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_fire, 'f', "Fire", "Fire which bolt? ", "You have no bolts to fire.",
			0, TV_BOLT, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATG | USE_HANDLE),
			(CONDITION_NO_WIND | CONDITION_SKILL_FIRE | CONDITION_NO_SNEAKING | CONDITION_SHOOTING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_fire, 'f', "Fire", "Fire which shot? ", "You have no shots to fire.",
			0, TV_SHOT, (USE_EQUIP | USE_INVEN | USE_FLOOR | USE_FEATG | USE_HANDLE),
			(CONDITION_NO_WIND | CONDITION_SKILL_FIRE | CONDITION_GUN_CHARGED | CONDITION_NO_SNEAKING | CONDITION_SHOOTING), TMD_CONDITION(TMD_BERSERK), 0, NULL},

	{ player_cast, 'm', "Cast spell", "Use which book? ", "You have nothing you have studied.",
			inven_cast_okay, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_ALLY | USE_HANDLE),
			(CONDITION_LITE | CONDITION_LITERATE | CONDITION_HOLD_SONG | CONDITION_NO_SNEAKING),
			TMD_CONDITION(TMD_BERSERK) | TMD_CONDITION(TMD_BLIND) | TMD_CONDITION(TMD_AMNESIA), 0, NULL},

	{ player_study, 'G', "Study", "Study which book? ", "You have no books that you can read.",
			inven_study_okay, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_ALLY | USE_HANDLE),
			(CONDITION_LITE | CONDITION_LITERATE | CONDITION_NEED_SPELLS),
			TMD_CONDITION(TMD_BERSERK) | TMD_CONDITION(TMD_BLIND) | TMD_CONDITION(TMD_AMNESIA), 0, NULL},

	{ player_browse, 'b', "Browse", "Browse which book? ", "You have no books that you can browse.",
			inven_book_okay, 0, (USE_INVEN | USE_FLOOR | USE_FEATU | USE_ALLY | USE_TARGET | USE_RANGE | USE_KNOWN | USE_HANDLE), (CONDITION_LITERATE), 0L, 0, NULL},
			
	{ player_offer, 'O', "Offer", "Offer which item? ", "You are not carrying anything to offer.",
			item_tester_hook_droppable, 0, (USE_INVEN | USE_BAGC | USE_BAGS | USE_GOLD | USE_HANDLE), (CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK) | TMD_CONDITION(TMD_CONFUSED), COMMAND_ITEM_TRADE, NULL},

	{ player_trade, 'T', "Trade", "Trade for which item? ", "You see nothing to trade.",
			item_tester_hook_tradeable, 0, (USE_TARGET | USE_RANGE | USE_GOLD | USE_HANDLE), (CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK) | TMD_CONDITION(TMD_CONFUSED), 0, NULL},

	{ player_steal, 's', "Steal", "Steal which item? ", "You see nothing to steal.",
			NULL, 0, (USE_TARGET | USE_GOLD), (CONDITION_NO_SNEAKING), TMD_CONDITION(TMD_BERSERK) | TMD_CONDITION(TMD_CONFUSED), 0, NULL},

	{ player_handle, 'h', "Handle", "Handle which item? ", "You see nothing to handle.",
			NULL, 0, (0xFFFFFFFFL) & ~(USE_HANDLE | USE_FEATH | USE_SKIN | USE_QUIVER), 0, 0, 0, NULL},
			
	{ player_set_trap_or_spike, 'S', "Spike/Set trap", "Spike/Set trap with which item? ", "You have nothing to set a trap or spike with.",
			NULL, 0, (USE_INVEN | USE_FLOOR | USE_BAGC | USE_ALLY | USE_HANDLE), 0L, 0L, 0, NULL},

};


/*
 * To design additional familiars, choose a name, blow type, and then 3 abilities:
 * the first two are gained immediately, the third when the player reaches level 10.
 * Look up the abilities in the familiar ability table below.
 */
const familiar_type familiar_race[MAX_FAMILIARS] =
{
	{"", 0, 0, 0, 0},
	{ "monkey",	RBM_HIT, 54, 45, FAMILIAR_SHOT },
	{ "cat",	RBM_BITE, 45, FAMILIAR_VISION, FAMILIAR_BLOW + GF_CONFUSION },
	{ "owl",	RBM_PECK, 46, FAMILIAR_VISION, 33 },
	{ "mouse",	RBM_BITE, 56, 72, FAMILIAR_SPEED },
	{ "duck", 	RBM_PECK, 46, 42, 85 },
	{ "snake", 	RBM_BITE, FAMILIAR_BLOW, 37, FAMILIAR_BLOW + GF_POIS },
	{ "wolverine", RBM_CLAW, 53, FAMILIAR_BLOW, 182},
	{ "badger",	RBM_BUTT, 34, FAMILIAR_BLOW, 225 },
	{ "rat",	RBM_BITE, 45, 42, FAMILIAR_BLOW + GF_DISEASE },
	{ "spider",	RBM_BITE, 45, 37, FAMILIAR_BLOW + GF_POIS },
	{ "mole",	RBM_CLAW, 34, 228, 51 },
	{ "ferret", RBM_BITE, 72, 228, FAMILIAR_SPEED },
	{ "bat", RBM_BITE, 46,  231, FAMILIAR_BLOW + GF_VAMP_DRAIN },
	{ "hawk", RBM_CLAW, 46,  FAMILIAR_VISION, FAMILIAR_SPEED },
	{ "pig", RBM_BUTT, 53, FAMILIAR_VISION, FAMILIAR_SIZE },
	{ "dog", RBM_BITE, 53, 228, 229 },
	{ "toad", RBM_TOUCH, 84, FAMILIAR_VISION, FAMILIAR_MANA },
	{ "frog", RBM_BITE, 42, 45, 166 },
	{ "goat", RBM_BUTT, 49, FAMILIAR_BLOW, FAMILIAR_BLOW + GF_FORCE},
	{ "turtle", RBM_BITE, 57, 42, 236 },
	{ "newt", RBM_BITE, 230, 42, FAMILIAR_MANA },
	{ "lizard", RBM_BITE, 53, 45, 166 },
	{ "crow", RBM_PECK, 46, 54, 33 },
	{ "penguin", RBM_PECK, 42, 53, 83},
	{ "parrot",	RBM_PECK, 46, 56, 33 },
	{ "dragonfly",	RBM_BITE, 46, 37, FAMILIAR_SPEED },	
	{ "moth",	RBM_BITE, 46, 37, 166 }
};


const familiar_ability_type familiar_ability[MAX_FAMILIAR_ABILITIES] =
{
	{"", 0, 999},

		/* Pick 2 - levels 2 & 4 */
	{"become armoured against arrows and acid", 57, 0},
	{"become hidden to infravision", 37, 0},
	{"climb walls and ceilings", 45, 0},
	{"dig through dirt and rubble", 34, 0},
	{"eat body parts to recover health", 53, 0},
	{"fit through narrow spaces", 72, 0},
	{"increase range of senses", FAMILIAR_VISION, 0},
	{"swim", 42, 0},
	{"take items from the ground", 54, 0},
	{"track enemies by scent", 228, 0},

		/* Pick 2 - levels 6 & 8 */
	{"act sneaky", 56, 0},
	{"bash through doors", 49, 0},
	{"become harder to hit", FAMILIAR_AC, 0},
	{"communicate with you silently", 74, 0},
	{"fly", 46, 45},
	{"open doors and disarm traps", 48, 54},
	{"radiate light", 35, 0},
	{"resist blindness and see invisible monsters", 231, 0},
	{"resist mental attack", 38, 0},
	{"track enemies by scent through water", 230, 42},

		/* Pick 2 - levels 10 & 12 */
	{"gain an additional attack", FAMILIAR_BLOW, 0},
	{"immunity to acid", 80, 0 },
	{"immunity to electricity", 81, 0},
	{"immunity to fire", 82, 0},
	{"immunity to cold", 83, 0},
	{"immunity to poison", 84, 0},
	{"increase size", FAMILIAR_SIZE, 0},
	{"increase speed", FAMILIAR_SPEED, 0},
	{"temporarily resist elemental damage", 184, 0},
	{"turn temporarily invisible", 166, 56},

		/* Pick 2 - levels 14 & 15 */
	{"bless itself", 181, 0},
	{"cause hunger", 185, 0},
	{"cause amnesia", 174, 0},
	{"create darkness", 172, 0},
	{"magically probe an opponent", 186, 0},
	{"make attack poisonous", FAMILIAR_BLOW + GF_POIS, FAMILIAR_BLOW},
	{"move invisibly", 36, 166},
	{"replace attack with firing spikes", FAMILIAR_SPIKE, FAMILIAR_BLOW},
	{"replace attack with throwing stones", FAMILIAR_SHOT, FAMILIAR_BLOW},
	{"track enemies unerringly by scent", 229, 228},

		/* Pick 2 - levels 16 & 18 */
	{"carry additional ammunition", 52, FAMILIAR_SHOT},
	{"cast stinking cloud", 132, 0},
	{"create traps", 173, 0},
	{"magically shield itself", 183, 0},
	{"grow additional ammunition", 100, FAMILIAR_SPIKE},
	{"cause fear", 187, 0},
	{"cause blindness", 188, 0},
	{"cause confusion", 189, 0},
	{"resist powerful light", 232, 0},
	{"resist powerful darkness", 233, 0},

		/* Pick 2 - levels 20 & 22 */
	{"become giant size", 73, FAMILIAR_SIZE},
	{"cast helpful spells on you", 58, 181},
	{"cure itself of ills", 163, 0},
	{"go berserk", 182, 0},
	{"give drained health to you", FAMILIAR_BLOW + GF_VAMP_DRAIN_FAMILIAR, FAMILIAR_BLOW + GF_VAMP_DRAIN},
	{"heal itself", 162, 0},
	{"haste itself", 160, 0},
	{"increase speed", FAMILIAR_SPEED, 0},
	{"increase spell attack damage", FAMILIAR_POWER, 0},
	{"make attack drain health", FAMILIAR_BLOW + GF_VAMP_DRAIN, FAMILIAR_BLOW},
	
		/* Pick 2 - levels 24 & 25 */
	{"become harder to hit", FAMILIAR_AC, 0},
	{"blink away", 164, 0},
	{"cast spells intelligently", 33, 0},
	{"increase hit points", FAMILIAR_HP, 0},
	{"increase mana", FAMILIAR_MANA, 0},
	{"increase size", FAMILIAR_SIZE, FAMILIAR_BLOW},
	{"never miss when attacking", 225, FAMILIAR_BLOW},
	{"teleport away", 165, 164},
	{"teleport enemies to itself", 168, 164},
	{"teleport itself to targets", 167, FAMILIAR_BLOW},

		/* Pick 2 - levels 26 & 28 */
	{"become harder to hit", FAMILIAR_AC, 0},
	{"cause slowness", 190, 0},
	{"cause illusions", 179,  0},
	{"cause wounds", 180, 0},
	{"cause paralysis", 191, 190},
	{"curse enemies", 176, 0},
	{"dispel magic on enemies", 177, 0},
	{"increase size", FAMILIAR_SIZE, FAMILIAR_BLOW},
	{"regenerate quickly", 41, 0},
	{"resist teleportation", 235, 0},

		/* Pick 2 - levels 30 & 32 */
	{"cast acid bolt", 144, 0},
	{"cast lightning bolt", 145, 0},
	{"cast fire bolt", 146, 0},
	{"cast frost bolt", 147, 0},
	{"cast poison bolt", 148, 0},
	{"increase mana", FAMILIAR_MANA, 0},
	{"resist nexus", 89, 0},
	{"resist disenchantment", 90, 0},
	{"resist chaos", 234, 0},
	{"resist lava", 87, 82},

		/* Pick 2 - levels 34 & 36 */
	{"become harder to hit", FAMILIAR_AC, 0},
	{"gain an additional attack", FAMILIAR_BLOW, 0},
	{"increase hit points", FAMILIAR_HP, 0},
	{"increase mana", FAMILIAR_MANA, 0},
	{"increase size", FAMILIAR_SIZE, 0},
	{"increase spell attack damage", FAMILIAR_POWER, 0},
	{"make attack diseased", FAMILIAR_BLOW + GF_DISEASE, FAMILIAR_BLOW + GF_POIS},
	{"make attack blinding", FAMILIAR_BLOW + GF_BLIND, 0},
	{"make attack confusing", FAMILIAR_BLOW + GF_CONFUSION, 0},
	{"make attack terrifying", FAMILIAR_BLOW + GF_TERRIFY, 0},

		/* Pick 2 - levels 38 & 40 */
	{"cast acid ball", 128, 0},
	{"cast lightning ball", 129, 0},
	{"cast fire ball", 130, 0},
	{"cast frost ball", 131, 0},
	{"cast ball of light", 133, 0},
	{"cast ball of darkness", 134, 0},
	{"cast mana bolt", 153, 0},
	{"resist plasma", 88, 0},
	{"resist nether", 86, 0},
	{"resist powerful water", 85, 0},

		/* Pick 2 - levels 41 & 42 */
	{"add mana to itself", 161, FAMILIAR_MANA},
	{"assume wraith form", 171, 0},
	{"breath acid",104, 80},
	{"breath electricity",105, 81},
	{"breath fire",106, 82},
	{"breath cold",107, 83},
	{"drain mana from enemies", 175, 0},
	{"increase spell attack damage", FAMILIAR_POWER, 0},
	{"resist edged weapons", 236, 0},
	{"resist blunt weapons", 237, 0},

    	/* Pick 2 - levels 43 & 44 */
	{"blast the minds of enemies", 178, 0},
	{"breath light",110, 232},
	{"breath darkness",111, 233},
	{"breath poison",108, 84},
	{"breath powerfully", 44, 73},
	{"cast plasma bolt", 149, 88},
	{"cast ice bolt", 150, 83},
	{"cast water bolt", 151, 85},
	{"cast nether bolt", 152, 86},
	{"teleport enemies away", 169, 0},

		/* Pick 2 - levels 45 & 46 */
	{"cast helpful spells on you", 58, 181},
	{"gain an additional attack", FAMILIAR_BLOW, 0},
	{"give drained mana to you", FAMILIAR_BLOW + GF_MANA_DRAIN_FAMILIAR, FAMILIAR_BLOW + GF_MANA_DRAIN},
	{"immunity to edged weapons", 238, 236},
	{"immunity to blunt weapons", 239, 237},
	{"increase spell attack damage", FAMILIAR_POWER, 0},
	{"make attack drain mana", FAMILIAR_BLOW + GF_MANA_DRAIN, FAMILIAR_BLOW},
	{"pass through walls", 50, 171},
	{"resistance to magic", 242, 0},
	{"tunnel through walls", 51, 34},

		/* Pick 2 - levels 47 & 48 */
	{"become harder to hit", FAMILIAR_AC, 0},
	{"breath chaos", 120, 0},
	{"breath fear", 126, 0},
	{"breath sound", 113, 0},
	{"breath shards", 114, 0},
	{"breath nexus", 118, 0},
	{"breath nether", 119, 0},
	{"cast lightning beam", 155, 0},
	{"cast ice beam", 156, 0},
	{"cast arc of force", 159, 0},

		/* Pick 2 - levels 49+ */
	{"breath disease", 127, 0},
	{"breath inertia", 114, 0},
	{"breath gravity", 115, 0},
	{"breath force", 117, 0},
	{"breath disenchantment", 123, 0},
	{"breath time", 124, 0},
	{"cast nether beam", 157, 0},
	{"cast arc of hellfire", 158, 0},
	{"increase hit points", FAMILIAR_HP, 0},
	{"increase spell attack damage", FAMILIAR_POWER, 0}
};

const timed_effect timed_effects[TMD_MAX] =
{
	{ "You feel temporarily stronger!", "You feel less strong.",
			NULL, NULL,
			"You are too strong.",
			"You are temporarily stronger.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily smarter!", "You feel less smart.",
			NULL, NULL,
			"You are too smart.",
			"You are temporarily smarter.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily wiser!", "You feel less wise.",
			NULL, NULL,
			"You are too wise.",
			"You are temporarily wiser.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily more dextrous!", "You feel less dextrous.",
			NULL, NULL,
			"You are too dextrous.",
			"You are temporarily more dextrous.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily healthier!", "You feel less healthy.",
			NULL, NULL,
			"You are too healthy.",
			"You are temporarily healthier.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily cuter!", "You feel less attractive.",
			NULL, NULL,
			"You are too cute.",
			"You are temporarily cuter",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily more agile!", "You feel less agile.",
			NULL, NULL,
			"You are too agile.",
			"You are temporarily more agile.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily larger!", "You feel smaller again.",
			NULL, NULL,
			"You are too large.",
			"You are temporarily larger.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily weaker!", "You feel less weak.",
			NULL, NULL,
			"You are too weak.",
			"You are temporarily weaker.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily stupider!", "You feel less stupid.",
			NULL, NULL,
			"You are too stupid.",
			"You are temporarily stupider.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily more naive!", "You feel less naive.",
			NULL, NULL,
			"You are too naive.",
			"You are temporarily more naive.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily more fumble fingered!", "You feel less fumble fingered.",
			NULL, NULL,
			"You are too fumble fingered.",
			"You are temporarily more fumble fingered.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily sicker!", "You feel less sick.",
			NULL, NULL,
			"You are too sickly.",
			"You are temporarily sick.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily uglier!", "You feel less ugly.",
			NULL, NULL,
			"You are too ugly.",
			"You are temporarily uglier.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily more clumsy!", "You feel less clumsy.",
			NULL, NULL,
			"You are too clumsy.",
			"You are temporarily more clumsy.",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel temporarily smaller!", "You feel larger again.",
			NULL, NULL,
			"You are too small.",
			"You are temporarily smaller",
			PR_STATS, PU_BONUS, MSG_GENERIC },
	{ "You feel yourself moving faster!", "You feel yourself slow down.",
			NULL, NULL,
			"You are moving too quickly.",
			"You are temporarily hasted.",
			PR_SPEED, PU_BONUS, MSG_SPEED },
	{ "You feel yourself moving slower!", "You feel yourself speed up.",
			NULL, NULL,
			"You are moving too slowly.",
			"You are temporarily slowed.",
			PR_SPEED, PU_BONUS, MSG_SLOW },
	{ "You are blind.", "You can see again.",
			NULL, NULL,
			"You can't see anything.",
			"You cannot see.",
			(PR_MAP | PR_BLIND), PU_FORGET_VIEW | PU_UPDATE_VIEW | PU_MONSTERS, MSG_BLIND },
	{ "You are paralysed!", "You can move again.",
			"You are more paralysed!", "You are less paralysed.",
			"You are paralysed!",
			"You are paralysed, which prevents you from any action.",
			PR_STATE, 0, MSG_PARALYZED },
	{ "You are confused!", "You are no longer confused.",
			"You are more confused!", "You feel a little less confused.",
			"You are confused.",
			"You are confused.",
			PR_CONFUSED, 0, MSG_CONFUSED },
	{ "You are terrified!", "You feel bolder now.",
			"You are more scared!", "You feel a little less scared.",
			"You are too terrified.",
			"You are terrified.",
			PR_AFRAID, PU_BONUS, MSG_AFRAID },
	{ "You feel drugged!", "You can see clearly again.",
			"You feel more drugged!", "You feel less drugged.",
			"You feel drugged.",
			"You are hallucinating.",
			PR_MAP, 0, MSG_DRUGGED },
	{ "You are poisoned!", "You are no longer poisoned.",
			"You are more poisoned!", "You are less poisoned.",
			"The poison prevents you.",
			"You are poisoned.",
			PR_POISONED, 0, MSG_POISONED },
	{ NULL, NULL, NULL, NULL, NULL, "You are bleeding.", 0, 0, 0 },  /* TMD_CUT -- handled seperately */
	{ NULL, NULL, NULL, NULL, NULL, "You are stunned.", 0, 0, 0 },  /* TMD_STUN -- handled seperately */
	{ "You feel unlucky.", "The curse has expired.",
			NULL, NULL,
			"Your luck has run out.",
			"Your luck has run out.",
			PR_CURSED, PU_BONUS, MSG_GENERIC },
	{ "You feel your memories fade.", "Your memories come flooding back.",
			NULL, NULL,
			"You have forgotten how.",
			"You are temporarily unable to remember things.",
			PR_AMNESIA, 0, MSG_GENERIC },
	{ "You are petrified to the spot!", "You can move freely again.",
			NULL, NULL,
			"You are too petrified!",
			"You are petrified to the spot.",
			PR_PETRIFY, 0, MSG_GENERIC },
	{ "You are stuck in a time-loop!", "You can move again.",
			NULL, NULL,
			"You are trapped in a temporal anomaly.",
			"You are trapped in a temporal anomaly.",
			PR_STATE, 0, MSG_GENERIC },
	{ "Your eyelids feel heavy.", "You no longer feel sleepy.",
			NULL, NULL,
			"You feel too drowsy.",
			"You are prone to fall asleep.",
			PR_STATE, 0, MSG_GENERIC },
	{ "You feel drowsy.", "You wake up.",
			NULL, NULL,
			"You feel too drowsy.",
			"You are about to fall asleep.",
			PR_STATE, 0, MSG_GENERIC },
	{ "You feel safe from evil!", "You no longer feel safe from evil.",
			"You feel even safer from evil!", "You feel less safe from evil.",
			"You are protected from evil.",
			"You are protected from evil.",
			0, 0, MSG_PROT_EVIL },
	{ "You fade from view!", "You feel visible once more.",
			NULL, NULL,
			"You cannot be seen.",
			"You cannot be seen.",
			0, PU_BONUS, MSG_INVULN },
	{ "You feel like a hero!", "You no longer feel heroic.",
			"You feel more like a hero!", "You feel less heroic.",
			"A hero wouldn't do this.",
			"You are heroic.",
			0, PU_BONUS, MSG_HERO },
	{ "You feel like a killing machine!", "You no longer feel berserk.",
			"You feel even more enraged!", "You feel less angry.",
			"You are too enraged!",
			"You are in a berserk rage.",
			0, PU_BONUS, MSG_BERSERK },
	{ "A mystic shield forms around your body!", "Your mystic shield crumbles away.",
			"The mystic shield strengthens.", "The mystic shield weakens.",
			"The mystic shield prevents you.",
			"You are protected by a mystic shield.",
			0, PU_BONUS, MSG_SHIELD },
	{ "You feel righteous!", "The prayer has expired.",
			"You feel more righteous!", "You feel less righteous.",
			"You are too righteous.",
			"You are temporarily blessed.",
			0, PU_BONUS, MSG_BLESSED },
	{ "Your eyes feel very sensitive!", "Your no longer feel so sensitive.",
			"Your eyes feel more sensitive!", "Your eyes feel less sensitive.",
			"Your eyes are too sensitve.",
			"You can see invisible.",
			0, (PU_BONUS | PU_MONSTERS), MSG_SEE_INVIS },
	{ "Your eyes begin to tingle!", "Your eyes stop tingling.",
			"Your eyes' tingling intensifies.", "Your eyes tingle less.",
			"Your eyes are too sensitive to heat.",
			"You can see further in the infrared spectrum.",
			0, (PU_BONUS | PU_MONSTERS), MSG_INFRARED },
	{ "You feel resistant to acid!", "You are no longer resistant to acid.",
			"You feel more resistant to acid!", "You feel less resistant to acid.",
			"You cannot be hurt by this acid.",
			"You are temporarily protected from acid.",
			0, 0, MSG_RES_ACID },
	{ "You feel resistant to electricity!", "You are no longer resistant to electricity.",
			"You feel more resistant to electricity!", "You feel less resistant to electricity.",
			"You cannot be hurt by this electricity.",
			"You are temporarily protected from electricity.",
			0, 0, MSG_RES_ELEC },
	{ "You feel resistant to fire!", "You are no longer resistant to fire.",
			"You feel more resistant to fire!", "You feel less resistant to fire.",
			"You cannot be hurt by this fire.",
			"You are temporarily protected from fire.",
			0, 0, MSG_RES_FIRE },
	{ "You feel resistant to cold!", "You are no longer resistant to cold.",
			"You feel more resistant to cold!", "You feel less resistant to cold.",
			"You cannot be hurt by this cold.",
			"You are temporarily protected from cold.",
			0, 0, MSG_RES_COLD },
	{ "You feel resistant to poison!", "You are no longer resistant to poison.",
			"You feel more resistant to poison!", "You feel less resistant to poison.",
			"You cannot be hurt by this poison.",
			"You are temporarily protected from poison.",
			0, 0, MSG_RES_POIS },
	{ "You feel resistant to lava!", "You are no longer resistant to lava.",
			"You feel more resistant to lava!", "You feel less resistant to lava.",
			"You cannot be hurt by this lava.",
			"You are temporarily protected from lava.",
			0, 0, MSG_GENERIC },
	{ "You feel resistant to water!", "You are no longer resistant to water.",
			"You feel more resistant to water!", "You feel less resistant to water.",
			"You cannot be hurt by this water.",
			"You are temporarily protected from water.",
			0, 0, MSG_GENERIC },
	{ "You feel resistant to confusion!", "You are no longer resistant to confusion.",
			"You feel more resistant to confusion!", "You feel less resistant to confusion.",
			"You cannot be hurt by this confusion.",
			"You are temporarily protected from confusion.",
			0, PU_BONUS, 0 },
	{ NULL, NULL,
			NULL, NULL,
			"You cannot yet be hurt by this poison.",
			"You have poison slowed in your veins.",
			PR_POISONED, 0, MSG_RES_POIS },
	{ NULL, NULL,
			NULL, NULL,
			"Your metabolism is too slow.",
			"Your metabolism is slowed.",
			0, 0, MSG_GENERIC },
	{ NULL, NULL,
			NULL, NULL,
			"You are recalling.",
			"You will soon be recalled.",
			PR_POISONED, 0, MSG_RES_POIS },
	{ NULL, NULL,
			NULL, NULL,
			"You are returning to a nearby location.",
			"You will soon be returned to a nearby location.",
			PR_POISONED, 0, MSG_RES_POIS },
	{ NULL, NULL,
			NULL, NULL,
			"You are delaying a spell.",
			"You have delayed casting of a spell.",
			0, 0, MSG_GENERIC },
	{ "You feel as if you move more freely!", "You are no longer moving as freely.",
			NULL, NULL,
			"You cannot be hurt by this.",
			"You are temporarily protected from paralysation and magical slowness.",
			0, PU_BONUS, 0 },
	{ "Your mind expands.", "Your horizons are once more limited.",
			"Your mind expands further.", NULL,
			"Your mind is too open to further experience.",
			"You are temporarily telepathic.",
			0, PU_BONUS, MSG_GENERIC },
	{ "Your skin turns to stone.", "A fleshy shade returns to your skin.",
			NULL, NULL,
			"Your skin is too rigid.",
			"Your skin is stone.",
			0, PU_BONUS, MSG_GENERIC },
	{ "You feel the need to run away, and fast!", "The urge to run dissipates.",
			NULL, NULL,
			"You are too terrified.",
			"You are fleeing in terror.",
			0, PU_BONUS, MSG_AFRAID },
	{ "You start sprinting.", "You suddenly stop sprinting.",
			NULL, NULL,
			"You are running too quickly",
			"You are sprinting.",
			0, PU_BONUS, MSG_SPEED },
};


const skill_table_entry skill_table[] =
{
		{SKILL_TO_HIT_MELEE, "Fighting", 12 },
		{SKILL_TO_HIT_BOW, "Shooting", 12 },
		{SKILL_TO_HIT_THROW, "Throwing", 12 },
		{SKILL_STEALTH, "Stealth", 1},
		{SKILL_SAVE, "Save Throw", 6},
		{SKILL_DISARM, "Disarming", 6},
		{SKILL_DEVICE, "Devices", 6},
		{SKILL_SEARCH, "Searching", 6},
		{SKILL_DIGGING, "Digging", 6},
		{-1, "", 0}
};


const char *inscrip_info[] =
{
		NULL,
		"It is a cursed artifact of some kind, that you may be able to enchant back to full strength.  ",
		"It is a cursed ego item of some kind.  ",
		"It is cursed, with negative effects if you attempt to use it.  ",
		"It is broken, having been blasted by powerful magic.  ",
		"It needs recharging.  ",
		"It is of good quality, but with no additional powers.  ",
		"It is a useful ego item.  ",
		"It is a useful artifact.  ",
		"It is not cursed.  It may be an ego item or artifact.  ",
		"It is of very good quality, but with no additional powers.  ",
		"It is of great quality, but with no additonal powers.  ",
		"It is a useful ego item, with a random hidden ability.  ",
		"It is an artifact that has resisted your destruction.  ",
		"It is an ego item or artifact that resisted being picked up or used.  ",
		"It is of average quality or worse, and may be cursed.  It may be an cursed ego item or artifact.  ",
		"It is better than average quality, and not cursed.  It may be an ego item or artifact.  ",
		"It is either better than average quality or cursed.  It is not an ego item or artifact.  ",
		"It is an ego item, but may or may not be cursed.  ",
		"It is an ego item, with a random hidden ability, but may or may not be cursed.  ",
		"It is an artifact, but may or may not be cursed.  ",
		"There are no runes on it.  ",
		"There are runes on it.  It may be an ego item or artifact.  ",
		"It is of average quality, but may be damaged by wear and tear.  ",
		"It is valuable, but may or may not be cursed.  ",
		"It is worth further investigation.  It may be a cursed item, ego item or artifact, or simply broken.  ",
		"It is coated with a substance.  ",
		"It has a magically applied enchantment.  "
};


